/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.basis;


/**
 * Ein Instanz dieser Klasse enthaelt Informationen darueber ob die Werte eines
 * Helden regelkonform sind.
 * Wenn ein Wert eines Helden nicht mit den Regeln uebereinstimmt koennen hier
 * Beschreibungen der Probleme gespeicher und abgerufen werden.
 */
public class KonfliktBeschreibung {

    /** Das oeffnende HTML-Tag. */
    private static final String HTML = "<html>";

    /** Der Html-Text der die Regelkonflikte beschreibt. */
    public final StringBuilder beschreibungen;

    /** Erzeugt eine neue KonfliktBeschreibung. */
    public KonfliktBeschreibung() {
        beschreibungen = new StringBuilder();
    }

    /**
     * Fuegt der KonfliktBeschreibung eine weiter Beschreibung eines Konflikt
     * hinzu.
     *
     * @param beschreibung eines Regelproblems. Die Beschreibung kann mit
     *      HTML-Auszeichnungen versehen sein, dabei ist das &lt;html&gt;-Tag
     *      wegzulassen, das wird automatisch hinzugefuegt.
     */
    public final void einfuegen( final String beschreibung ) {
        beschreibungen.append( beschreibung );
        beschreibungen.append( "<br>\n" );
    }

    /**
     * Haengt die Beschreibungen der uebergebenen KonfliktBeschreibung an diese
     * an.
     *
     * @param andereBeschreibung die an diese angehaengt werden soll.
     */
    public final void einfuegen(
            final KonfliktBeschreibung andereBeschreibung ) {

        beschreibungen.append( andereBeschreibung.beschreibungen );
    }

    /**
     * @return {@code true} wenn das Objekt Informationen ueber einen
     *      Regelkonflikt enthaelt, sonst {@code false}.
     */
    public final boolean enthaeltKonflikt() {
        return ( 0 < beschreibungen.length() );
    }

    /** Loescht alle Eintrage aus der Beschreibung. */
    public final void leeren() {
        beschreibungen.setLength( 0 );
    }

    /**
     * @return ein mit HTML-Auszeichnungen versehener String mit den
     *      Beschreibungen der aufgetretenen Regelkonflikte.
     *      Der String hat die Form:
     *      <pre>
     * &lt;html&gt;<i>Erste Beschreibung</i>&lt;br&gt;
     * <i>Zweite Beschreibung</i>&lt;br&gt;
     * ...
     *      </pre>
     *      Wenn {@code enthaeltKonflikt() == false} gibt diese Methode nur
     *      "&lt;html&gt;" zurueck.
     */
    public final String toHtml() {
        return HTML + beschreibungen.toString();
    }

    /**
     * @return ein mit HTML-Auszeichnungen versehener String mit den
     *      Beschreibungen der aufgetretenen Regelkonflikte. Im Gegensatz zu
     *      {@link #toHtml()} fehlt hier das oeffnende HTML-Tag.
     *      Der String hat die Form:
     *      <pre>
     * <i>Erste Beschreibung</i>&lt;br&gt;
     * <i>Zweite Beschreibung</i>&lt;br&gt;
     * ...
     *      </pre>
     *      Wenn {@code enthaeltKonflikt() == false} liefert diese Methode einen
     *      leeren String.
     */
    public final String toHtml2() {
        return beschreibungen.toString();
    }

    @Override
    public String toString() {
        return beschreibungen.toString().replace( "<br>", "" );
    }
}
