/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.basis;

import java.text.Collator;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Sprache;
import de.bernhardjung.dsaprogramm.talente.Talent;

/**
 * Eine Variante fuer die Sonderfertigkeit Kulturkunde.
 * Die Variante enthaelt den Namen der Kultur und die Sprachen, die
 * in der Kultur gesprochen werden.
 *
 * @author Bernhard Jung
 */
public class Kulturkunde implements Comparable< Kulturkunde > {

    /** Eine Liste aller Kulturkunden. */
    private static final List< Kulturkunde > KULTURKUNDEN =
        new ArrayList< Kulturkunde >();

    /** Eine Map in der jede Kulturkunde unter ihrer Id gespeichert ist. */
    private static final Map< String, Kulturkunde > KULTURKUNDEN_MAP =
        new HashMap< String, Kulturkunde >();

    public static final void initialisieren() {

        new Kulturkunde( "Kulturkunde.Almada", Talent.GARETHI );

        new Kulturkunde( "Kulturkunde.Amazonen", Talent.GARETHI );

        new Kulturkunde( "Kulturkunde.Ambosszwerge", Talent.ROGOLAN );

        new Kulturkunde( "Kulturkunde.AndergastNostria", Talent.GARETHI );

        new Kulturkunde( "Kulturkunde.Aranien", Talent.GARETHI,
                                                Talent.TULAMIDYA );

        new Kulturkunde( "Kulturkunde.ArchaischeAchaz", Talent.RSSAHH );

        new Kulturkunde( "Kulturkunde.Auelfen", Talent.ISDIRA );

        new Kulturkunde( "Kulturkunde.Bornland", Talent.GARETHI );

        new Kulturkunde( "Kulturkunde.Brilliantzwerge", Talent.ROGOLAN );

        new Kulturkunde( "Kulturkunde.Bukanier", Talent.GARETHI );

        new Kulturkunde( "Kulturkunde.Erzzwerge", Talent.ROGOLAN );

        new Kulturkunde( "Kulturkunde.Ferkina", Talent.TULAMIDYA );

        new Kulturkunde( "Kulturkunde.Firnelfen", Talent.ISDIRA );

        new Kulturkunde( "Kulturkunde.Fjarninger", Talent.THORWALSCH );

        new Kulturkunde( "Kulturkunde.Gjalskerlaender", Talent.THORWALSCH );

        new Kulturkunde( "Kulturkunde.Goblins", Talent.GOBLINISCH );

        new Kulturkunde( "Kulturkunde.Grolme", Talent.GARETHI );

        new Kulturkunde( "Kulturkunde.Horasreich", Talent.GARETHI );

        new Kulturkunde( "Kulturkunde.Huegelzwerge", Talent.ROGOLAN );

        new Kulturkunde( "Kulturkunde.Nivesen", Talent.NUJUKA );

        new Kulturkunde( "Kulturkunde.Norbarden", Talent.NUJUKA,
                                                  Talent.GARETHI);

        new Kulturkunde( "Kulturkunde.NordaventurischeStaedte",
                Sprache.GARETHI );

        new Kulturkunde( "Kulturkunde.Maraskan", Talent.GARETHI );

        new Kulturkunde( "Kulturkunde.Mohas", Talent.MOHISCH );

        new Kulturkunde( "Kulturkunde.Mittelreich", Talent.GARETHI );

        new Kulturkunde( "Kulturkunde.Novadi", Talent.TULAMIDYA );

        new Kulturkunde( "Kulturkunde.Orks", Talent.OLOARKH,
                                             Talent.OLOGHAIJAN );

        new Kulturkunde( "Kulturkunde.SchwarzeLande", Talent.GARETHI );

        new Kulturkunde( "Kulturkunde.StammesAchaz", Talent.RSSAHH );

        new Kulturkunde( "Kulturkunde.Steppenelfen", Talent.ISDIRA );

        new Kulturkunde( "Kulturkunde.Suedaventurien", Talent.GARETHI );

        new Kulturkunde( "Kulturkunde.Svellttal", Talent.GARETHI );

        new Kulturkunde( "Kulturkunde.Thorwal", Talent.THORWALSCH );

        new Kulturkunde( "Kulturkunde.Tocamuyac", Talent.MOHISCH );

        new Kulturkunde( "Kulturkunde.Trolle", Talent.GARETHI );

        new Kulturkunde( "Kulturkunde.Trollzacker", Talent.GARETHI );

        new Kulturkunde( "Kulturkunde.Tulamidenlande", Talent.TULAMIDYA );

        new Kulturkunde( "Kulturkunde.Waldelfen", Talent.ISDIRA );

        new Kulturkunde( "Kulturkunde.Zahori", Talent.TULAMIDYA );

        new Kulturkunde( "Kulturkunde.Zyklopeninseln", Talent.GARETHI );

        Collections.sort( KULTURKUNDEN );
    }

    /**
     *
     * @param id der gesuchten Kulturkunde.
     * @return die Kulturkunde mit der uebergebenen Id.
     * @throws IllegalArgumentException wenn es keine Kulturkunde mit der
     *      uebergebenen Id gibt.
     */
    public static final Kulturkunde getKulturkunde( final String id ) {

        if ( KULTURKUNDEN_MAP.containsKey( id ) ) {

            return KULTURKUNDEN_MAP.get( id );

        } else {

            throw new IllegalArgumentException(
                    "Es existiert keine Kulturkunde mit der Id " + id );

        }
    }

    /**
     * @return eine Liste aller Kulturkunden.
     */
    public static final List< Kulturkunde > getKulturkunden() {
        return new ArrayList< Kulturkunde >( KULTURKUNDEN );
    }

    /** Ein Collator zum alphabetischen Sortieren. */
    private static final Collator collator = Collator.getInstance();

    /** Die Id der Kulturkunde. */
    private final String id;

    /** Der Name der Kultur. */
    private final String name;

    /** Die Sprachen die in der Kultur gesprochen werden. */
    private final List< Sprache > sprachen;

    /**
     * @param dieId der Kulturkunde.
     * @param dieSprachen die in der Kultur gesprochen werden.
     */
    private Kulturkunde( final String dieId, final Sprache ... dieSprachen ) {

        id = dieId;

        name = I18n.i18n( id );

        sprachen = Arrays.asList( dieSprachen );

        KULTURKUNDEN.add( this );

        KULTURKUNDEN_MAP.put( id, this );
    }

    /**
     * Sortiert die Kulturkunden alphabetisch nach Name.
     */
    public final int compareTo( final Kulturkunde o ) {
        return collator.compare( name, o.name );
    }

    /** @return die Id der Kulturkunde. */
    public final String getId() {
        return id;
    }

    /**
     * @return eine Liste der in der Kultur gesprochenen Sprachen.
     */
    public final List< Sprache > getSprachen() {
        return new ArrayList< Sprache >( sprachen );
    }

    /**
     * @return den Namen der Kultur(kunde), also 'Mohas' oder
     *      'Nordaventurische Staedte'.
     */
    @Override
    public String toString() {
        return name;
    }
}
