/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.bausteine;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.*;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;

import de.bernhardjung.dsaprogramm.bausteine.bonus.SonderfertigkeitWertBonus;
import de.bernhardjung.dsaprogramm.bausteine.moeglicheprof.MoeglicheProfessionen;
import de.bernhardjung.dsaprogramm.bausteine.moeglicheprof.MoeglicheProfessionenAlleAusser;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.talente.Sprache;

/**
 * Repraesentiert eine Kultur.
 *
 * @author Bernhard Jung
 */
public class Kultur extends Heldenbaustein {

    /**
     * Eine Kultur die keine Boni vergibt und keine Kosten verursacht.
     * Sie kann als Platzhalter dienen wenn fuer einen Helden noch keine
     * Kultur gewaehlt wurde.
     */
    public static final Kultur LEERE_KULTUR = new Kultur( "Keine Kultur" );

    /**
     * Die Leitsonderfertigkeiten die man durch diese Kultur erhaelt.
     */
    private final List< SonderfertigkeitWertBonus > leitsonderfertigkeiten;

    /**
     * Die Anzahl zusaetzlicher Leitzauber die sich ein elfischer Held
     * aussuchen darf.
     */
    private int leitzauber;

    /** Die von der Kultur vergebene Muttersprache. */
    private Sprache muttersprache;

    /** Die moeglichen Professionen der Kultur. */
    private MoeglicheProfessionen professionen;

    /** Die Zweitsprachen aus denen man eine waehlen kann. */
    private final List< Sprache > zweitsprachen;

    /**
     * Erzeugt eine neue Kultur.
     *
     * @param id der Kultur.
     */
    @SuppressWarnings( "unchecked" )
    public Kultur( final String id ) {
        super( id, 7.5 );

        leitsonderfertigkeiten = new ArrayList< SonderfertigkeitWertBonus >();

        leitzauber = 0;

        professionen = new MoeglicheProfessionenAlleAusser(
                Collections.EMPTY_LIST );

        zweitsprachen = new ArrayList< Sprache >( 2 );
    }

    @Override
    public double berechneGpKosten() {

        double kosten = super.berechneGpKosten();

        if ( 0 < zweitsprachen.size() ) {

            kosten += 0.7;
        }

        if ( enthaelt( Sonderfertigkeit.KULTURKUNDE ) ) {

            kosten -= 1.5;
        }

        return kosten;
    }

    @Override
    IXMLElement erzeugeXmlWurzel() {
        return new XMLElement( TAG_KULTUR );
    }

    /**
     * @see #getAnzahlLeitzauber()
     * @return {@code true}, wenn der (elfische) Held zusaetzliche Leitzauber
     *         waehlen darf.
     */
    public final boolean hatZusaetzlicheLeitzauber() {
        return ( 0 != leitzauber );
    }

    /**
     * @return Die Leitsonderfertigkeiten der Kultur.
     */
    public final List< SonderfertigkeitWert > getLeitsonderfertigkeiten() {

        final List< SonderfertigkeitWert > werte =
            new ArrayList< SonderfertigkeitWert >(
                    leitsonderfertigkeiten.size() );

        for ( final SonderfertigkeitWertBonus wert : leitsonderfertigkeiten ) {

            werte.add( wert.getWert() );
        }

        return werte;
    }
    /**
     * @return die Anzahl zusaetzlicher Leitzauber, die ein elfischer Held noch
     *         waehlen darf.
     */
    public final int getAnzahlLeitzauber() {
        return leitzauber;
    }

    /**
     * @return Das Objekt das entscheidet welche Professionen fuer die Kultur
     *         moegliche sind.
     */
    public final MoeglicheProfessionen getMoeglicheProfessionen() {
        return professionen;
    }

    /**
     * @return die Muttersprache der Kultur.
     */
    public final Sprache getMuttersprache() {
        return muttersprache;
    }

    /**
     * @see #hatZweitsprache()
     * @return die Zweitsprachen der Kultur von denen der Held eine waehlen
     *      kann. Wenn die Kultur keine Zweitsprache besitzt ist die Liste
     *      leer.
     */
    public final List< Sprache > getZweitsprachen() {
        return new ArrayList< Sprache >( zweitsprachen );
    }

    /**
     * @return {@code true} wenn die Kultur eine Zweitsprache besitzt, sonst
     *      {@code false}.
     */
    public final boolean hatZweitsprache() {
        return 0 != zweitsprachen.size();
    }

    /**
     * @param anzahl
     *            der zusaetzlichen Leitzauber, die ein elfischer Held waehlen
     *            darf.
     */
    public final void setAnzahlLeitzauber( final int anzahl ) {
        leitzauber = anzahl;
    }

    /**
     * @param leitsf die Leitsonderfertigkeiten fuer die Kultur.
     */
    public final void setLeitsonderfertigkeiten(
            final List< SonderfertigkeitWertBonus > leitsf ) {

        leitsonderfertigkeiten.clear();

        leitsonderfertigkeiten.addAll( leitsf );
    }

    /**
     * @param moegliche
     *            das Objekt das entscheidet, welche Professionen als moegliche
     *            gelten.
     */
    public final void setProfessionen(
            final MoeglicheProfessionen moegliche ) {

        professionen = moegliche;
    }

    /**
     * @param dieMuttersprache der Kultur.
     */
    public final void setMuttersprache( final Sprache dieMuttersprache ) {

        muttersprache = dieMuttersprache;
    }

    /**
     * @param dieZweitsprachen der Kultur, von denen ein Held eine waehlen
     *      darf.
     */
    public final void setZweitsprachen( final List< Sprache > dieZweitsprachen ) {

        zweitsprachen.clear();

        zweitsprachen.addAll( dieZweitsprachen );
    }

    @Override
    public IXMLElement toXml() {

        final IXMLElement wurzelElement = super.toXml();

        // Der Name
        // ********
        if ( nameNichtLokalisiert( ".Name", getName() ) ) {

            final IXMLElement nameElement = new XMLElement( TAG_NAME );

            nameElement.setContent( getName() );

            wurzelElement.addChild( nameElement );
        }

        // Muttersprache
        // *************
        final IXMLElement mutterspracheElement =
            new XMLElement( TAG_MUTTERSPRACHE );

        mutterspracheElement.setContent( muttersprache.getId() );

        wurzelElement.addChild( mutterspracheElement );

        // Zweitsprache
        // ************
        if ( hatZweitsprache() ) {

            final IXMLElement zweitsprachenElement =
                new XMLElement( TAG_ZWEITSPRACHEN );

            for ( final Sprache sprache : zweitsprachen ) {

                final IXMLElement spracheElement = new XMLElement( TAG_SPRACHE);

                spracheElement.setContent( sprache.getId() );

                zweitsprachenElement.addChild( spracheElement );
            }

            wurzelElement.addChild( zweitsprachenElement );
        }

        // Leitzauber
        // **********
        if ( hatZusaetzlicheLeitzauber() ) {

            final IXMLElement leitzauberElement =
                new XMLElement( TAG_LEITZAUBER );

            leitzauberElement.setContent( String.valueOf( leitzauber ) );

            wurzelElement.addChild( leitzauberElement );
        }

        // Leitsonderfertigkeiten
        // **********************
        if ( 0 != leitsonderfertigkeiten.size() ) {

            final IXMLElement sfenElement = new XMLElement( TAG_LEITSF );

            for ( final SonderfertigkeitWertBonus wert :
                leitsonderfertigkeiten ) {

                sfenElement.addChild( wert.toXml() );
            }

            wurzelElement.addChild( sfenElement );
        }

        // moegliche Professionen
        // **********************
        wurzelElement.addChild( professionen.toXml() );

        return wurzelElement;
    }

}
