/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.bausteine.bonus;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.*;

import java.util.List;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;

import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;

/**
 * Eine Sonderfertigkeit die in einer Variante verbilligt wird.
 * Die Variante die verbilligt wird kann vom Benutzer festgelegt werden.
 *
 * @author Bernhard Jung
 */
public class VerbilligteVariante {

    /** Die Sonderfertigkeit die verbilligt wird. */
    private final Sonderfertigkeit sonderfertigkeit;

    /**
     * Die genaue Variante wird ja noch vom Benutzer festgelegt daher ist
     * es eigentlich nicht noetig explizit eine Anzugeben.
     * Im Fall der Ortskenntnis kann die Variante hier aber ein String mit
     * einem Erklaerungstext sein wie z.B. "eine ueblicherweise benutzte
     * Strecke"
     */
    private final Object variante;

    /**
     * Die Id, die in der XML-Datei fuer die Variante verwendet wird, oder ein
     * Text der als Variante fungiert.
     */
    private final String variantenId;

    /** Eine Beschreibung wie "Eine weitere Kulturkunde". */
    private final String beschreibung;

    /**
     * Der Text der Beschreibung oder die Id die, in der XML-Datei, fuer die
     * Variante verwendet wird.
     */
    private final String beschreibungId;

    /**
     *
     * @param dieSonderfertigkeit
     *            die verbilligt wird.
     * @param dieVariante
     *            eine Erklaerungstext der fuer die Variante eingesetzt wird.
     *            Dies ist natuerlich nur moeglich wenn es sich, wie z.B. bei
     *            Ortskenntnis um eine freie Variante handelt. Hier kann ein
     *            Text wie &quot;eine ueblicherweise benutzte Strecke&quot;
     *            uebergeben werden. Es kann hier {@code null} uebergeben werden
     *            in solch einem Fall ist aber eine Beschreibung zu uebergeben.
     *            Es kann hier auch eine Id fuer einen Text aus der
     *            Lokalisierungsdatei uebergeben werden.
     * @param dieBeschreibung
     *            der Verbilligung z.B. &quot;eine weiter Kulturkunde&quot; Wenn
     *            fuer {@code dieVariante} ein Text uebergeben wurde kann hier
     *            {@code null} uebergeben werden. Es kann hier auch eine Id,
     *            fuer einen Text aus der Lokalisiserungsdatei, uebergeben
     *            werden.
     * @throws IllegalArgumentException
     *             wenn fuer {@code dieVariante} und
     *             {@code dieBeschreibung null} uebergeben wurde.
     */
    public VerbilligteVariante(
            final Sonderfertigkeit dieSonderfertigkeit,
            final String dieVariante,
            final String dieBeschreibung ) {

        if ( ( null == dieVariante ) && ( null == dieBeschreibung ) ) {
            throw new IllegalArgumentException(
                    "dieVariante und dieBeschreibung sind null. " +
                    "Eines der Argumente muss ungleich null sein." );
        }

        sonderfertigkeit = dieSonderfertigkeit;

        variantenId = dieVariante;

        if ( null == dieVariante ) {

            final List< Object > varianten = sonderfertigkeit.getVarianten(null);

            if ( 0 < varianten.size() ) {

                variante = varianten.get( 0 );

            } else  {

                variante = "";
            }

        } else {

            if ( I18n.hatLokalisierungstext( dieVariante ) ) {

                variante = I18n.i18n( dieVariante );

            } else {

                variante = dieVariante;
            }

        }

        beschreibungId = dieBeschreibung;

        if ( null != dieBeschreibung ) {

            if ( I18n.hatLokalisierungstext( dieBeschreibung ) ) {

                beschreibung = I18n.i18n( dieBeschreibung );

            } else {

                beschreibung = dieBeschreibung;
            }

        } else {

            beschreibung = new SonderfertigkeitWert(
                    sonderfertigkeit,
                    variante ).toString();
        }
    }

    /**
     * @return die Id der Beschreibung, oder {@code null}, wenn es keine Id fuer
     *         eine Beschreibung gibt.
     */
    public final String getBeschreibungId() {
        return beschreibungId;
    }

    public double getGpKosten() {
        return sonderfertigkeit.getApKosten() / 250.0;
    }

    /**
     * @return die Id der Variante, oder {@code null}, wenn es keine Id fuer
     *         eine Variante gibt.
     */
    public final String getVariantenId() {
        return variantenId;
    }

    /**
     * @return Die Sonderfertigkeit die verbilligt wird.
     */
    public final SonderfertigkeitWert getWert() {
        return new SonderfertigkeitWert( sonderfertigkeit, variante );
    }

    /**
     * @return eine Beschreibung der Verbilligung wie &quot;eine weitere
     *         Kulturkunde&quot;
     */
    @Override
    public final String toString() {
        return beschreibung;
    }

    /**
     * @return das XML-Element, das die verbilligte Variante repraesentiert.
     */
    public final IXMLElement toXml() {

        final IXMLElement element = new XMLElement( TAG_VARIANTE );

        element.setAttribute( ATTR_SONDERFERTIGKEIT, sonderfertigkeit.getId() );

        if ( sonderfertigkeit.hatTalent() ) {

            element.setAttribute( ATTR_TALENT,
                                  sonderfertigkeit.getTalent().getId() );
        }

        if ( null != beschreibungId ) {

            element.setAttribute( ATTR_BESCHREIBUNG, beschreibungId );
        }

        if ( null != variantenId ) {

            element.setAttribute( ATTR_VARIANTE, variantenId );
        }

        return element;
    }
}
