/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.bausteine.util;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.*;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Wuerfel;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

/**
 * Wuerfelt das Alter eines Helden aus. Die Formel hat die Form: x + Wuerfel.
 * Wobei x eine Anzahl von Jahren ist und Wuerfel ein W6, W20 oder aehnliches.
 *
 * @author Bernhard Jung
 *
 */
public class AlterFormel {

    /** Eine Beschreibung der Formel. */
    private static final String BESCHREIBUNG = I18n.i18n( "Alter" );

    /**
     * Die Jahre die der Held mindestens alt sein muss. Dazu wird dann noch
     * ein Wuerfelwurf addiert.
     */
    private final int basis;

    /** Die Formel fuer das Alter als String. */
    private final String toString;

    /**
     * Der Wuerfel der zu Bestimmung des Alters geworfen wird.
     */
    private final Wuerfel wuerfel;

    /**
     * @param basisJahre die zum Wuerfelwurf dazu addiert werden.
     * @param w Der Wuerfel der geworfen wird um das Alter zu bestimmen.
     */
    public AlterFormel( final int basisJahre, final Wuerfel w ) {

        basis = basisJahre;

        wuerfel = w;

        toString = String.format( BESCHREIBUNG, basis, wuerfel );
    }

    /**
     * Wuerfelt das Alter eines Helden aus.
     * Beachtet dabei alle Sonderfaelle wie Gebildet und Zeitaufwaendig.
     *
     * @param held dessen Alter bestimmt werden soll.
     * @return ein moegliches Alter fuer den Helden.
     */
    public final int auswuerfeln( final Held held ) {

        int alter = basis + wuerfel.wuerfeln();

        if ( held.hat( VorNachteil.GEBILDET ) ) {

            final int stufe = held.getWert( VorNachteil.GEBILDET ).getStufe();

            alter += stufe / 2;
        }

        if ( held.getProfession().istZeitaufwaendig()
          || held.hat( VorNachteil.AKADEMISCHE_AUSBILDUNG_GELEHRTER ) ) {

            alter += 3;
        }

        return alter;
    }

    /** @return die Basisjahre zu denen ein Wuerfelwurf addiert wird. */
    public final int getBasis() {
        return basis;
    }

    /**
     * @return der Wuerfel, der zur Bestimmung des Alters geworfen wird.
     */
    public final Wuerfel getWuerfel() {
        return wuerfel;
    }

    @Override
    public final String toString() {
        return toString;
    }

    /**
     * @return ein XML-Element, das die Formel repraesentiert.
     */
    public final IXMLElement toXml() {

        final IXMLElement element = new XMLElement( TAG_ALTER );

        element.setAttribute( ATTR_BASIS, String.valueOf( basis ) );

        element.setAttribute( ATTR_WUERFEL,
                              String.valueOf( wuerfel.getAugenzahl() ) );

        return element;
    }
}
