/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.eigenschaften;

import java.util.ArrayList;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.basis.SKT;

/**
 * Repraesentiert den Eigenschaftswert eines Helden.
 *
 * @author Bernhard Jung
 */
public class EigenschaftWert {

    /**
     * Die Basisstsufe der Eigenschaft. Ueblicherweise die Anzahl der GP die
     * auf die Eigenschaft verteilt wurden. Nicht so beim SO.
     */
    private int basisstufe;

    /** Die Eigenschaft die zu dem Wert gehoert. */
    private final Eigenschaft eigenschaft;

    /** Der Held dem der Wert gehoert. */
    private final Held held;

    /**
     * Die AP-Kosten die aufgebracht wurden um den Wert zu steigern. An Position
     * i in der Liste stehen die AP die bezahlt wurden um den i-ten Punkt
     * hinzuzukaufen.
     */
    private final List< Integer > kosten;

    /** Gibt an ob der Held eine Spezielle Erfahrung erhalten hat. */
    private boolean spezielleErfahrung;

    /** Die Punkte, die der Benutzer mit AP zugekauft hat. */
    private int zukauf;

    /** Die zugekauften Punkte, die der Held direkt nach dem Laden besitzt. */
    private int zukaufVorgegeben;

    /**
     * Erzeugt einen neuen Eigenschafts-Wert.
     * Die Basisstufe des Wertes ist 0.
     *
     * @param derHeld dem der Wert gehoert. {@code null} ist nicht erlaubt.
     * @param dieEigenschaft des Wertes. {@code null} ist nicht erlaubt.
     */
    public EigenschaftWert(
            final Held derHeld,
            final Eigenschaft dieEigenschaft ) {

        if ( null == derHeld ) {
            throw new IllegalArgumentException(
                    "null ist als Argument fuer derHeld nicht erlaubt."
                    );
        }

        if ( null == dieEigenschaft ) {
            throw new IllegalArgumentException(
                    "null ist als Argument fuer dieEigenschaft nicht erlaubt."
                    );
        }

        held = derHeld;

        basisstufe = 0;

        eigenschaft = dieEigenschaft;

        kosten = new ArrayList< Integer >();
        kosten.add( 0 );

        spezielleErfahrung = false;

        zukauf = 0;

        zukaufVorgegeben = 0;
    }

    /**
     * @return die Basisstufe des Wertes, sie enthaelt keine Modifikationen.
     */
    public final int getBasisstufe() {
        return basisstufe;
    }

    /**
     * @return die Eigenschaft auf die sich der Wert bezieht.
     */
    public final Eigenschaft getEigenschaft() {
        return eigenschaft;
    }

    /**
     * @return die Modifikation welche die Eigenschaft durch Rasse, Kultur,
     *      Profession und Vor-/Nachteile erhaelt.
     */
    public final int getModifikation() {
        return eigenschaft.getModifikation( held );
    }

    /**
     * @return Die Stufe, die die Eigenschaft direkt nach der Erschaffung hatte.
     */
    public final int getStartstufe() {
        return getBasisstufe() + getModifikation();
    }

    /**
     * @param lernmethode
     *            die zum Steigern verwendet werden soll. Hat der Held eine
     *            Spezielle Erfahrung auf den Wert, so wird dieser Parameter
     *            ignoriert und automatisch die Lernmethode Spezielle Erfahrung
     *            verwendet.
     * @return die AP-Kosten, die zu entrichten sind um den Wert um einen Punkt
     *         zu steigern.
     */
    public final int getSteigerungskosten( final Lernmethode lernmethode ) {

        final int steigerungsfaktor;

        if ( spezielleErfahrung ) {

            steigerungsfaktor = Lernmethode.SPEZIELLE_ERFAHRUNG
                                .bestimmeSteigerungsfaktor( eigenschaft );

        } else {

            steigerungsfaktor =
                lernmethode.bestimmeSteigerungsfaktor( eigenschaft );
        }

        return SKT.getKosten( ( 1 + getStufe() ), steigerungsfaktor );
    }

    /**
     * @return die gesamte Stufe des Werts, d.h. Basisstufe + Modifikation.
     */
    public final int getStufe() {
        return getBasisstufe() + getModifikation() + zukauf;
    }

    /**
     * @return die Punkte die durch AP hinzugekauft wurden.
     */
    public final int getZukauf() {
        return zukauf;
    }

    /**
     * @return {@code true}, wenn der Held eine spezielle Erfahrung auf den Wert
     *         hat, sonst {@code false}.
     */
    public final boolean hatSpezielleErfahrung() {
        return spezielleErfahrung;
    }

    /**
     * Ueberprueft ob der Wert den Regeln entspricht und liefert eine
     * Beschreibung evtl auftretender Regelkonflikte.
     *
     * @return einer Auflistung der bestehenden Regelprobleme.
     */
    public final KonfliktBeschreibung pruefen() {
        return eigenschaft.pruefen( held, this );
    }

    /* *
     * Senkt die zugekauften Punkte um einen Punkt und schreibt dem Helden
     * die frei gewordenen AP gut. Wenn die zugekauften Punkte gleich 0 sind, so
     * macht die Methode nichts.
     * /
    public final void senken() {

        if ( zukauf > 0 ) {

            held.addiereApEingesetzt( - kosten.remove( zukauf ) );

            --zukauf;

            held.notifyObservers( eigenschaft );
        }
    }
*/

    /**
     * @param neueBasisstufe des Wertes.
     */
    public final void setBasisstufe( final int neueBasisstufe ) {

        if ( neueBasisstufe != basisstufe ) {

            basisstufe = neueBasisstufe;

            held.notifyObservers( eigenschaft );
        }
    }

    /**
     * @param erfahrung
     *            {@code true}, wenn der Held eine spezielle Erfahrung auf den
     *            Wert erhalten hat, sonst {@code false}.
     */
    public final void setSpezielleErfahrung( final boolean erfahrung ) {

        if ( erfahrung != spezielleErfahrung ) {

            spezielleErfahrung = erfahrung;

            held.notifyObservers( eigenschaft );
        }

    }

    /**
     * Stellt die Stufe des Wertes ein, indem die Basisstufe angepasst wird.
     *
     * @param stufe
     *            die der Wert besitzen soll.
     */
    public void setStufe( final int stufe ) {

        if ( stufe != getStufe() ) {

            final int modifikation = getModifikation();

            basisstufe = ( stufe - modifikation - zukauf );

            held.notifyObservers( eigenschaft );
        }
    }

    /**
     * Stellt die zugekauften Punkte ein ohne dem Helden AP-Kosten zu berechnen.
     *
     * @param punkte
     *            die der Held zugekauft hat.
     */
    public final void setZukauf( final int punkte ) {

        if ( punkte != zukauf ) {

            while( kosten.size() <= punkte ) {

                kosten.add( 0 );
            }

            while( kosten.size() > ( punkte + 1 ) ) {

                kosten.remove( kosten.size() - 1 );
            }

            zukauf = punkte;

            held.notifyObservers( eigenschaft );
        }
    }

    /**
     * Aendert die zugekauften Punkte. Aktualisiert die AP des Helden
     * entsprechend.
     *
     * @param punkte
     *            auf die die zugekauften Punkte eingestellt werden sollen.
     * @param lernmethode
     *            falls die zugekauften Punkte erhoeht werden. Hat der Held eine
     *            Spezielle Erfahrung auf den Wert, so wird dieser Parameter
     *            ignoriert und automatisch die Lernmethode Spezielle Erfahrung
     *            verwendet. Die Spezielle Erfahrung wird aber nur einmal
     *            verwendet. Soll der Wert z.B. von 1 auf 3 angehoben werden, so
     *            wird die Steigerung auf 2 mit Spezieller Erfahrung gemacht und
     *            die Steigerung auf 3 mit der uebergebenen Lernmethode.
     */
    public final void setZukauf(
            final int punkte,
            final Lernmethode lernmethode ) {

        if ( zukauf < punkte ) {

            int summe = 0; // die Gesamtkosten fuer die Steigerung

            for ( ; zukauf < punkte ; ++zukauf ) {

                final int apKosten;

                if ( zukauf < zukaufVorgegeben ) {

                    apKosten = 0;

                } else {

                    apKosten = getSteigerungskosten( lernmethode );
                }

                spezielleErfahrung = false;

                kosten.add( apKosten );

                summe += apKosten;
            }

            held.addiereApEingesetzt( summe );

            held.notifyObservers( eigenschaft );

        } else if ( zukauf > punkte ) {

            int summe = 0;

            for ( ; zukauf > punkte ; --zukauf ) {

                summe += kosten.remove( zukauf );
            }

            held.addiereApEingesetzt( -summe );

            held.notifyObservers( eigenschaft );
        }
    }

    /**
     * Stellt die zugekauften Punkte ein, die der Held direkt nach dem Laden des
     * Helden besitzt. Stellt die eigentlichen zugekauften Punkte auf den selben
     * Wert ein.
     *
     * @param punkte
     *            die der Held zur Eigenschaft zugekauft hat.
     */
    public final void setZukaufVorgegeben( final int punkte ) {

        if ( punkte > zukauf ) {

            for( ; zukauf < punkte ; ++zukauf ) {

                kosten.add( 0 );
            }

            zukaufVorgegeben = punkte;

            held.notifyObservers( eigenschaft );

        } else if ( punkte < zukauf ){

            for ( ; zukauf > punkte ; --zukauf ) {

                kosten.remove( zukauf );
            }

            zukaufVorgegeben = punkte;

            held.notifyObservers( eigenschaft );
        }

    }
    /* *
     * Hebt den Wert um einen Punkt an. Erhoeht die zugekauften Punkte. Stellt
     * dem Helden die Kosten in Rechnung.
     *
     * @param lernmethode die zum Steigern verwendet werden soll.
     * /
    public final void steigern( final Lernmethode lernmethode ) {

        final int apKosten = getSteigerungskosten( lernmethode );

        kosten.add( apKosten );

        held.addiereApEingesetzt( apKosten );

        ++zukauf;

        spezielleErfahrung = false;

        held.notifyObservers( eigenschaft );
    }
    */
}
