/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.factories.ButtonBarFactory;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.i18n.I18n;

/**
 * Zeigt eine Fehlermeldung an, wenn eine Exception geworfen wird.
 * Die Klasse muss mittels
 * {@code System.setProperty( "sun.awt.exception.handler",
 *              "de.bernhardjung.dsaprogramm.gui.DialogFehlermeldung" ) }
 * als Exceptionhandler angemeldet werden.
 *
 * @author Bernhard Jung
 */
public final class DialogFehlermeldung extends JDialog {

    /** Der Titel fuer den Dialog. */
    private static final String FEHLERMELDUNG_TITEL =
        I18n.i18n( "DialogFehlermeldung.Titel" );
    
    /** Der Text der Fehlermeldung die der Benutzer angezeigt bekommt. */
    private static final String FEHLERMELDUNG = 
        I18n.i18n( "DialogFehlermeldung.Meldung" );

    /** 
     * Die Aufschrift Schliessen fuer den Knopf mit dem das Fenster geschlossen
     * wird. 
     */
    private static final String SCHLIESSEN = I18n.i18n( "Knopf.Schliessen" );
    
    /** Die Taste mit der die Aktion augefuehrt werden kann. */
    private static final String SCHLIESSEN_MNEMONIC = 
        I18n.i18n( "Knopf.Schliessen.Mnemonic" );
    
    /**
     * Zeigt dem Benutzer eine Fehlermeldung an.
     * 
     * @param meldung ein Satz der den Fehler genauer spezifiziert.
     * @param t die geworfene Exception. Darf nicht {@code null} sein.
     */
    public static final void zeigeDialog( 
            final String meldung, 
            final Throwable t ) {
        
        final DialogFehlermeldung dialog = new DialogFehlermeldung();
        
        dialog.addMeldung( meldung );
        
        dialog.handle( t );
    }

    /**
     * Zeigt dem Benutzer eine Fehlermeldung an.
     * 
     * @param meldung ein Satz der den Fehler genauer spezifiziert.
     */
    public static final void zeigeDialog( final String meldung ) {
        
        final DialogFehlermeldung dialog = new DialogFehlermeldung();
        
        dialog.addMeldung( meldung );
        
        dialog.setVisible( true );
    }

    /**
     * Zeigt dem Benutzer eine Fehlermeldung an.
     * 
     * @param t die geworfene Exception. Darf nicht {@code null} sein.
     */
    public static final void zeigeDialog( final Throwable t ) {
        
        final DialogFehlermeldung dialog = new DialogFehlermeldung();
        
        dialog.handle( t );
    }

    /** 
     * Ein Textfeld, das den ganzen Text enthaelt, der im Fenster 
     * dargestellt wird. 
     */
    private final JTextArea taFehlermeldung;
    
    public DialogFehlermeldung() {
   
        super();
        
        setTitle( FEHLERMELDUNG_TITEL );
        setDefaultCloseOperation( JFrame.DISPOSE_ON_CLOSE );
        setSize( new Dimension( 320, 240 ) );
        setLocationRelativeTo( null );
        
        taFehlermeldung = new JTextArea();
        taFehlermeldung.setEditable( false );
        taFehlermeldung.setLineWrap( true );
        taFehlermeldung.setWrapStyleWord( true );
        taFehlermeldung.setBackground( Color.WHITE );
        
        final FormLayout layout = new FormLayout( "F:D:G", "F:D:G, 4dlu, D" );
        final DefaultFormBuilder builder = new DefaultFormBuilder( layout );
        builder.setDefaultDialogBorder();
        
        builder.append( new JScrollPane( taFehlermeldung ) );
        
        builder.nextLine( 2 );
        builder.append( erzeugePanelSchliessen() );
        
        add( builder.getPanel() );
        
        println( FEHLERMELDUNG );
        
        println( "Java Version: " + System.getProperty( "java.version" ) );
        println( "Betriebssystem: " + System.getProperty( "os.name" ) );
        println( "Version: " + System.getProperty( "os.version" ) );
        println( "Architektur: " + System.getProperty( "os.arch" ) );
    }

    /**
     * Fuegt der Fehlermeldung eine weitere Zeile hinzu.
     * Die Methode sollte am besten vor {@link #handle(Throwable)} verwendet 
     * werden, damit der Text vor dem stack trace erscheint.
     * 
     * @param meldung die an die Fehlermeldung angehaengt werden soll.
     */
    public final void addMeldung( final String meldung ) {
       println( meldung ); 
    }
    
    /** 
     * Wird automatisch aufgerufen wenn eine Exception geworfen wurde.
     * Die Methode fuegt die Meldung der Exception und den Stacktrace ins
     * Fenster ein. Anschliessend macht sie das Fenster sichtbar.
     */
    public final void handle( final Throwable t ) {
        
        println( t.toString() );
        for ( StackTraceElement ste : t.getStackTrace() ) {
        
            println( ste.toString() );
            
        }
        
        taFehlermeldung.setCaretPosition( 0 );
        
        setVisible(true);
        toFront();
    }
    
    /**
     * Fuegt eine weitere Zeile ins Fenster ein.
     * Die Zeile wird an das {@link #tfFehlermeldung Textfeld} angehaengt.
     * 
     * @param zeile Die naechste Zeile die eingefuegt werden soll.
     */
    private final void println( final String zeile ) {
        taFehlermeldung.append( zeile );
        taFehlermeldung.append("\n");
    }
    
    /*
    private final void copyToClipboard() {
     
        final Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
        final TransferHandler transferHandler = taFehlermeldung.getTransferHandler();
     
        taFehlermeldung.setSelectionStart( 0 );
        taFehlermeldung.setSelectionEnd( taFehlermeldung.getText().length() );
     
        transferHandler.exportToClipboard( taFehlermeldung, clipboard, TransferHandler.COPY );
     
        taFehlermeldung.setSelectionEnd( 0 );
    }
     */
    
    /**
     * @return ein Panel mit einem Knopf um das Fenster zu schliessen.
     */
    private final JPanel erzeugePanelSchliessen() {
        
        final JButton button = new JButton( SCHLIESSEN );
        
        button.addActionListener( new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setVisible( false );
                dispose();
            }
        } );
        
        button.setMnemonic( Util.getKeyCode( SCHLIESSEN_MNEMONIC ) );
        
        return ButtonBarFactory.buildCloseBar( button );
    }
}
