/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui;

import static de.bernhardjung.dsaprogramm.i18n.I18n.i18n;

import java.awt.Color;
import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.eigenschaften.EigenschaftWert;
import de.bernhardjung.dsaprogramm.i18n.I18n;

/**
 * Ein Panel zum Anzeigen und Editieren der Eigenschaftswerte eine Helden.
 *
 * @author Bernhard Jung
 */
class PanelEigenschaften extends JPanel implements Observer {

    /**
     * Der Text mit dem angegeben wird wie viele Punkte schon auf die
     * Eigenschaften verteilt worden sind.
     */
    private static final String VERTEILT =
        I18n.i18n( "PanelEigenschaften.verteilt" );

    /**
     * Die Label die den Namen der Eigenschaft anzeigen.
     * Die Label in der Liste sind den Eigenschaften ueber ihren Index
     * zugeordnet d.h. an Position i steht das Label fuer die Eigenschaft mit
     * index i.
     */
    private final List< JLabel > eigenschaften;

    /**
     * Die Label die die Gesamtstufe der Eigenschaften anzeigen.
     * Die Label in der Liste sind den Eigenschaften ueber ihren Index
     * zugeordnet d.h. an Position i steht das Label fuer die Eigenschaft mit
     * index i.
     */
    private final List< JLabel > gesamtstufen;

    /**
     * Die Label die die Modifikation der Eigenschaften anzeigen.
     * Die Label in der Liste sind den Eigenschaften ueber ihren Index
     * zugeordnet d.h. an Position i steht das Label fuer die Eigenschaft mit
     * index i.
     */
    private final List< JLabel > modifikationen;

    /**
     * Das Label mit dem angezeigt wird wie viel Punkte schon auf die
     * Eigenschaften verteilt worden sind.
     */
    private final JLabel verteilt;

    /**
     * Die Spinner mit denen die Startstufe der Eigenschaft festgelegt wird.
     * Die Spinner in der Liste sind den Eigenschaften ueber ihren Index
     * zugeordnet d.h. an Position i steht der Spinner fuer die Eigenschaft mit
     * index i.
     */
    private final List< JSpinner > basisstufen;

    /**
     * Die Objekte die den Wert aus den Spinnern mit den Werten des Helden
     * synchronisieren.
     * Die Synchronisierer in der Liste sind den Eigenschaften/Spinnern ueber
     * ihren Index zugeordnet d.h. an Position i steht der Synchronisierer fuer
     * die Eigenschaft mit index i.
     */
    private final List< WertSynchronisierer > wertSynchronisierer;

    /** Erzeugt das Panel und ordnet die Komponenten darauf an. */
    PanelEigenschaften() {

        eigenschaften = new ArrayList< JLabel >( 9 );

        modifikationen = new ArrayList< JLabel >( 9 );

        gesamtstufen = new ArrayList< JLabel >( 9 );

        verteilt = new JLabel( VERTEILT );

        verteilt.setHorizontalAlignment( SwingConstants.CENTER );

        basisstufen = new ArrayList< JSpinner >( 9 );

        wertSynchronisierer = new ArrayList< WertSynchronisierer >( 9 );

        final Eigenschaft[] eigenschaften;
        eigenschaften = Eigenschaft.getEigenschaftenMitSo();

        for ( final Eigenschaft eigenschaft : eigenschaften ) {

            this.eigenschaften.add( new JLabel( eigenschaft.toString() ) );

            modifikationen.add( Util.erzeugeLabelRechtsbuendig() );

            gesamtstufen.add( Util.erzeugeLabelRechtsbuendig() );

            final JSpinner spinner = Util.erzeugeSpinner( 0, 0, 99, 1 );
            basisstufen.add( spinner );

            final WertSynchronisierer synchronisierer =
                new WertSynchronisierer( spinner );

            spinner.addChangeListener( synchronisierer );

            wertSynchronisierer.add( synchronisierer );

        }

        final FormLayout layout = new FormLayout(
                //Eigensch.    Mod.     +    Spinner    =      Wert
                "7dlu, P:G, 4dlu, P:G, 4dlu, P, 4dlu, P:G, 4dlu, P, 2dlu, P:G, 7dlu",
                "7dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, " +
                "4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 7dlu");

        setLayout( layout );

        final CellConstraints cc = new CellConstraints();

        add( verteilt, cc.xyw( 2, 2, 11 ) );

        // Ueberschriften der Spalten
        add( Util.erzeugeUeberschrift(
                i18n( "PanelEigenschaften.Eigenschaft" ) ),
             cc.xy(  2, 4 ) );
        add( Util.erzeugeUeberschrift( i18n( "PanelEigenschaften.Mod" ) ),
             cc.xy(  4, 4 ) );
        add( Util.erzeugeUeberschrift( i18n( "PanelEigenschaften.Basisstufe" )),
             cc.xy(  8, 4 ) );
        add( Util.erzeugeUeberschrift( i18n( "PanelEigenschaften.Aktuell" ) ),
             cc.xy( 12, 4 ) );

        // Die restlichen Zeilen aufbauen
        for ( int i = 0 ; i < eigenschaften.length ; ++i ) {

            final int y = 2 * i + 6;
            int x = 0;

            add( this.eigenschaften.get( i ), cc.xy( x+=2, y ) );
            add( modifikationen.get( i ),     cc.xy( x+=2, y ) );
            add( new JLabel( "+" ),           cc.xy( x+=2, y ) );
            add( basisstufen.get( i ),        cc.xy( x+=2, y ) );
            add( new JLabel( "=" ),           cc.xy( x+=2, y ) );
            add( gesamtstufen.get( i ),       cc.xy( x+=2, y ) );
        }
    }

    /**
     * Bestimmt den Helden der gerade erschaffen wird.
     *
     * @param held der gerade erschaffen wird.
     */
    public final void setHeld( final Held held ) {

        for ( final Eigenschaft eigenschaft :
            Eigenschaft.getEigenschaftenMitSo() ){

            final int index = eigenschaft.getIndex();

            wertSynchronisierer.get( index ).setWert(
                    held.getWert( eigenschaft ) );
        }

        held.addObserver( this );

        update( held, null );
    }

    /**
     * Aktualisiert die angezeigten Werte.
     * Veraenderte Modifikationen werden uebernommen, ebenso die veraenderte
     * Gesamstufe der Eigenschaften.
     * Regelkonflikte werden angezeigt.
     *
     * @param o der ueberwachte Held.
     * @param arg ungenutzt.
     */
    public void update( final Observable o, final Object arg ) {

        final Held held = (Held) o;

        final int verteiltePunkte = held.getVerteilteEigenschaftspunkte();

        final int verteilbar = Eigenschaft.getMaxVerteilbar( held );

        verteilt.setText( String.format( VERTEILT,
                                              verteiltePunkte,
                                              verteilbar ) );

        if ( verteilbar < verteiltePunkte ) {

            verteilt.setForeground( Util.FEHLERFARBE );

        } else {

            verteilt.setForeground( getForeground() );

        }

        for ( final Eigenschaft eigenschaft :
            Eigenschaft.getEigenschaftenMitSo() ){

            final int index = eigenschaft.getIndex();

            final EigenschaftWert wert = held.getWert( eigenschaft );

            final KonfliktBeschreibung beschreibung = wert.pruefen();

            if( beschreibung.enthaeltKonflikt() ) {

                eigenschaften.get( index ).setToolTipText(
                        beschreibung.toHtml() );

                eigenschaften.get( index ).setForeground( Color.RED );

            } else {

                eigenschaften.get( index ).setToolTipText( null );

                eigenschaften.get( index ).setForeground( getForeground() );

            }

            modifikationen.get( index ).setText(
                    String.valueOf( wert.getModifikation() ) );


            final JSpinner spinner = basisstufen.get( index );

            spinner.setValue( held.getWert( eigenschaft ).getBasisstufe() );


            gesamtstufen.get( index ).setText(
                    String.valueOf( wert.getStufe() ) );

        }
    }

    /**
     * Synchronisiert einen JSpinner und einen {@link EigenschaftWert}.
     * Lauscht auf Aenderungen am Spinner und uebertraegt den Wert aus
     * dem Spinner als Startstufe in den {@link EigenschaftWert}.
     * */
    private class WertSynchronisierer implements ChangeListener {

        /** Der Spinner mit dem der EigenschaftWert synchronisiert wird. */
        private final JSpinner spinner;

        /** In den der Wert aus dem Spinner eingetragen wird. */
        private EigenschaftWert wert;

        /**
         * @param derSpinner dessen Wert ausgelesen wird.
         */
        private WertSynchronisierer( final JSpinner derSpinner ) {
            spinner = derSpinner;
        }

        /**
         * Uebertraegt den Wert aus dem Spinner in den EigenschaftWert.
         * @param e ungenutzt
         */
        public void stateChanged( final ChangeEvent e ) {

            if ( null != wert ) {

                wert.setBasisstufe( (Integer) spinner.getValue() );
            }
        }

        /**
         * Aendert den Wert in den der Wert aus dem Spinner eingetragen wird.
         * Setzt den Spinner auf die Startstufe aus dem EigenschaftWert
         *
         * @param neuerWert der aktualisert werden soll.
         */
        public void setWert( final EigenschaftWert neuerWert ) {

            wert = neuerWert;

            spinner.setValue( wert.getBasisstufe() );
        }
    }
}
