/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui;

import static de.bernhardjung.dsaprogramm.i18n.I18n.i18n;

import java.awt.Component;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;

import javax.swing.DefaultListCellRenderer;
import javax.swing.JLabel;
import javax.swing.JList;

import de.bernhardjung.dsaprogramm.bausteine.Kultur;
import de.bernhardjung.dsaprogramm.bausteine.bonus.Talentbonus;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.util.Strings;

/**
 * Erlaubt es eine Kultur zu waehlen.
 *
 * @author Bernhard Jung
 */
class PanelKultur extends PanelHeldenbaustein< Kultur > {

    /**
     * Gibt dem Benutzer an wie viele zusaetzliche Leitzauber der Held waehlen
     * darf.
     */
    private static final String LEITZAUBER =
        I18n.i18n( "PanelKultur.Leitzauber" );

    /**
     * Der Text 'Muttersprache' fuer das Anzeigen der Muttersprache einer
     * Kultur.
     * Beispiel: 'Muttersprache Garethi'
     */
    private static final String MUTTERSPRACHE =
        I18n.i18n( "PanelKultur.Muttersprache" );

    /**
     * Erklaert dem Benutzer, dass die Kultur in Verbindunge mit der
     * gewaehlten Rasse laut Regeln nicht erlaubt ist.
     */
    private static final String NICHT_MOEGLICH =
        i18n( "PanelKultur.KulturUndRasseNichtKombinierbar" );

    /**
     * Der Text 'Professionen' fuer die Aufzaehlung der moeglichen Professionen.
     */
    private static final String PROFESSIONEN =
        I18n.i18n( "PanelKultur.Professionen" );

    /**
     * Der Text 'Zweitsprache' fuer das Anzeige der Zweitsprache einer Kultur.
     * Beispiel: 'Zweitsprache Garethi'
     */
    static final String ZWEITSPRACHE =
        I18n.i18n( "PanelKultur.Zweitsprache" );

    /**
     * Die Kulturen die zur gewaehlten Rasse gehoeren.
     * Laut Regeln darf man mit bestimmten Rassen nur bestimmte Kulturen
     * kombinieren.
     */
    private final Collection< Kultur > moeglicheKulturen;

    /** Das Panel ueber das die Professionen gewaehlt werden. */
    private final PanelProfession panelProfession;

    /**
     * Erzeugt das Panel.
     *
     * @param kulturen die zur Auswahl stehen.
     */
    PanelKultur( final PanelProfession dasPanelProfession ) {

        super();
        moeglicheKulturen = new HashSet< Kultur >();
        for ( final Kultur kultur : getBausteine() ) {
            moeglicheKulturen.add( kultur );
        }

        panelProfession = dasPanelProfession;

        setListCellRenderer( new KulturListCellRenderer() );
    }

    /** {@inheritDoc} */
    @Override
    boolean bausteineGeaendert( final Object arg ) {
        return Datenspeicher.KULTUREN.equals( arg );
    }

    /** {@inheritDoc} */
    @Override
    final Kultur[] getBausteine() {
        final List< Kultur > kulturen =
            Datenspeicher.getInstanz().getKulturen();

        return kulturen.toArray( new Kultur[ kulturen.size() ] );
    }

    /** {@inheritDoc} */
    @Override
    final String getInformationen( final Kultur kultur ) {

        final StringBuilder informationen = new StringBuilder(
                "<html><body>");

        // *****************************
        // Ueberschrift
        // *****************************
        informationen.append( "<p><b>" );
        informationen.append( kultur.getName() );
        informationen.append( "</b></p>" );

        informationen.append( "<p><b>" );
        informationen.append( GENERIERUNGSKOSTEN );
        informationen.append( ": </b>" );
        informationen.append( kultur.getGpKosten( held ) );
        informationen.append( " " );
        informationen.append( GP );
        informationen.append( "</p>" );

        // *****************************
        // Voraussetzungen
        // *****************************
        appendVoraussetzungen( informationen, kultur.getVoraussetzungen() );

        // *****************************
        // Modifikationen
        // *****************************
        appendModifikationen(informationen, kultur.getModifikationen(), kultur);

        // *****************************
        // Automatische Vor-/Nachteile
        // *****************************
        appendAutomatischeVorNachteile(
                informationen,
                kultur.getVorteile(),
                kultur.getNachteile(),
                kultur.getVerbindungen() );

        // *************************
        // Empfohlene Vor-/Nachteile
        // *************************
        appendEmpfohleneVorNachteile(
                informationen,
                kultur.getEmpfohleneVorteile(),
                kultur.getEmpfohleneNachteile() );

        // **************************
        // Ungeeignete Vor-/Nachteile
        // **************************
        appendUngeeigneteVorNachteile(
                informationen,
                kultur.getUngeeigneteVorteile(),
                kultur.getUngeeigneteNachteile() );

        // *****************************
        // Moegliche Professionen
        // *****************************
        informationen.append( "<p><b>" );
        informationen.append( PROFESSIONEN );
        informationen.append( ":</b> " );

        informationen.append( kultur.getMoeglicheProfessionen() );

        informationen.append( "</p>" );

        // *****************************
        // Talente
        // *****************************
        informationen.append( "<br><p><b>" );
        informationen.append( TALENTE );
        informationen.append( "</b></p>" );

        for (final Talentgruppe talentgruppe: Talentgruppe.getTalentgruppen()){
            informationen.append( "<p><b>" );
            informationen.append( talentgruppe.getKuerzel() );
            informationen.append( ": </b>" );

            final List< Talentbonus > boni =
                kultur.getTalentboni( talentgruppe );

            if ( Talentgruppe.SPRACHEN == talentgruppe ) {
                informationen.append( MUTTERSPRACHE );
                informationen.append( " "  );
                informationen.append( kultur.getMuttersprache() );
                informationen.append( " ("  );
                informationen.append( Eigenschaft.KL.getKuerzel()  );
                informationen.append( "-2)"  );

                if ( kultur.hatZweitsprache() ) {
                    informationen.append( "; " );
                    informationen.append( ZWEITSPRACHE );
                    informationen.append( " "  );
                    informationen.append(  Strings.erzeugeAufzaehlungMitOder(
                            kultur.getZweitsprachen() ) );
                }

                if ( 0 < boni.size() ) {
                    informationen.append( "; " );
                }
            }

            if ( 0 < boni.size() ) {

                final String aufzaehlung;
                aufzaehlung = Strings.erzeugeAufzaehlung( boni );

                informationen.append( aufzaehlung );

            } else if ( Talentgruppe.SPRACHEN != talentgruppe ) {

                informationen.append( "-" );

            }

            informationen.append( "</p>" );
        }

        if ( kultur.hatLeittalente() ) {

            informationen.append( "<p><b>" );
            informationen.append( LEITTALENTE );
            informationen.append( ":</b> " );

            final String aufzaehlung;
            aufzaehlung = Strings.erzeugeAufzaehlung( kultur.getLeittalente());
            informationen.append( aufzaehlung );
            informationen.append( "</p>" );
        }

        // ******
        // Zauber
        // ******
        appendZauber(
                informationen,
                kultur.getZauberboni() );

        // *****************************
        // Leitzauber
        // *****************************
        if ( kultur.hatZusaetzlicheLeitzauber() ) {

            informationen.append( "<p>" );

            informationen.append( String.format(
                    LEITZAUBER,
                    kultur.getAnzahlLeitzauber() ) );

            informationen.append( "</p>" );
        }

        // *****************************
        // Sonderfertigkeiten
        // *****************************
        appendSonderfertigkeiten(
                informationen,
                kultur.getSonderfertigkeiten() );

        // ******************************
        // verbilligte Sonderfertigkeiten
        // ******************************
        appendVerbilligteSonderfertigkeiten(
                informationen,
                kultur.getVerbilligteSonderfertigkeiten(),
                kultur.getVerbilligteVarianten() );

        informationen.append( "</body></html>");

        return informationen.toString();
    }

    /**
     * Aendert die Kultur des zu erschaffenden Helden.
     *
     * @param neueKultur des Helden.
     */
    @Override
    final void setGewaehlterBaustein( final Kultur neueKultur ) {

        if ( null != neueKultur ) {

            held.setKultur( neueKultur );
            panelProfession.setMoeglicheProfessionen(
                    neueKultur.getMoeglicheProfessionen() );

        } else {

            held.setKultur( Kultur.LEERE_KULTUR );
            panelProfession.setMoeglicheProfessionen(
                    Kultur.LEERE_KULTUR.getMoeglicheProfessionen() );

        }
    }

    /**
     * Bestimmt die zur aktuell gewaehlten Rasse gehoerenden Kulturen.
     */
    public final void setMoeglicheKulturen(
            final Collection< Kultur > kulturen ) {

        moeglicheKulturen.clear();
        moeglicheKulturen.addAll( kulturen );
    }

    private class KulturListCellRenderer extends DefaultListCellRenderer {

        @Override
        public Component getListCellRendererComponent(
                final JList list,
                final Object value,
                final int index,
                final boolean isSelected,
                final boolean cellHasFocus ) {

            final JLabel label = (JLabel) super.getListCellRendererComponent(
                    list,
                    value,
                    index,
                    isSelected,
                    cellHasFocus );

            if ( moeglicheKulturen.contains( value ) ) {

                label.setToolTipText( null );

            } else {

                label.setForeground( Util.FEHLERFARBE );
                label.setToolTipText( NICHT_MOEGLICH );

            }

            return label;
        }

    }
}
