/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui;

import java.awt.Component;
import java.util.Collections;
import java.util.List;

import javax.swing.DefaultListCellRenderer;
import javax.swing.JLabel;
import javax.swing.JList;

import de.bernhardjung.dsaprogramm.bausteine.Profession;
import de.bernhardjung.dsaprogramm.bausteine.bonus.Talentbonus;
import de.bernhardjung.dsaprogramm.bausteine.moeglicheprof.MoeglicheProfessionen;
import de.bernhardjung.dsaprogramm.bausteine.moeglicheprof.MoeglicheProfessionenAlleAusser;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.util.Strings;

/**
 * Benutzerschnittstelle um eine Kultur zu waehlen.
 *
 * @author Bernhard Jung
 */
class PanelProfession extends PanelHeldenbaustein< Profession > {

    /** Der Text fuer den besonderen Besitz. */
    private static final String BESONDERER_BESITZ =
        I18n.i18n( "PanelProfession.BesondererBesitz" );

    /**
     * Erklaert dem Benutzer, dass die Profession in Verbindung mit der
     * gewaehlten Kultur laut Regeln nicht erlaubt ist.
     */
    private static final String NICHT_MOEGLICH =
        I18n.i18n( "PanelProfession.NichtKombinierbar" );

    /** Markiert eine Profession als Zeitaufwaendig. */
    private static final String ZEITAUFWENDIG =
        I18n.i18n( "PanelProfession.Zeitaufwendig" );

    /** Die der Kultur nach moeglichen Professionen. */
    private MoeglicheProfessionen moeglicheProfessionen;

    /**
     * Erzeugt das Panel.
     *
     * @param professionen die zur Auswahl stehen.
     */
    @SuppressWarnings( "unchecked" )
    PanelProfession() {
        super();

        moeglicheProfessionen = new MoeglicheProfessionenAlleAusser(
                Collections.EMPTY_LIST );

        setListCellRenderer( new ProfessionListCellRenderer() );
    }

    /** {@inheritDoc} */
    @Override
    boolean bausteineGeaendert( final Object arg ) {
        return Datenspeicher.PROFESSIONEN.equals( arg );
    }

    /** {@inheritDoc} */
    @Override
    Profession[] getBausteine() {
        final List< Profession > professionen =
            Datenspeicher.getInstanz().getProfessionen();

        return professionen.toArray( new Profession[ professionen.size() ] );
    }

    /** {@inheritDoc} */
    @Override
    final String getInformationen( final Profession profession ) {

        final StringBuilder informationen =
            new StringBuilder( "<html><body>" );


        informationen.append( "<p><b>" );

        final String nameMaennlich = profession.getName();
        final String nameWeiblich = profession.getNameWeiblich();

        informationen.append( nameMaennlich );

        // In der englischen Version sind die Namen gleich. In diesem Fall muss
        // nur ein Name angezeigt werden.
        if ( ! nameMaennlich.equals( nameWeiblich ) ) {
            informationen.append( " / " );
            informationen.append( nameWeiblich );
        }

        informationen.append( " (" );
        informationen.append( profession.getGpKosten( held ) );
        informationen.append( " " );
        informationen.append( GP );
        if ( profession.istZeitaufwaendig() ) {
            informationen.append( ", " );
            informationen.append( ZEITAUFWENDIG );
        }
        informationen.append( ")</b></p>" );

        // *****************************
        // Voraussetzungen
        // *****************************
        appendVoraussetzungen( informationen, profession.getVoraussetzungen() );

        // *****************************
        // Modifikationen
        // *****************************
        final List< Object > modifikationen = profession.getModifikationen();
        appendModifikationen( informationen, modifikationen, profession );

        // *****************************
        // Automatische Vor-/Nachteile
        // *****************************
        appendAutomatischeVorNachteile(
                informationen,
                profession.getVorteile(),
                profession.getNachteile(),
                profession.getVerbindungen() );


        // *****************************
        // Talente
        // *****************************
        for (final Talentgruppe talentgruppe: Talentgruppe.getTalentgruppenAlle()){

            final List< Talentbonus > boni =
                profession.getTalentboni( talentgruppe );

            if ( Talentgruppe.GABEN.equals( talentgruppe )
              || ( Talentgruppe.RITUALKENNTNIS.equals( talentgruppe )
                   && ( 0 == boni.size() ) ) ) {

                continue;
            }

            informationen.append( "<p><b>" );
            informationen.append( talentgruppe.getKuerzel() );
            informationen.append( ":</b> " );

            if ( 0 < boni.size() ) {

                final String aufzaehlung;
                aufzaehlung = Strings.erzeugeAufzaehlung( boni );

                informationen.append( aufzaehlung );

            } else {

                informationen.append( "-" );
            }

            informationen.append( "</p>" );
        }

        // *****************************
        // Sonderfertigkeiten
        // *****************************
        appendSonderfertigkeiten(
                informationen,
                profession.getSonderfertigkeiten() );

        // ******************************
        // verbilligte Sonderfertigkeiten
        // ******************************
        appendVerbilligteSonderfertigkeiten(
                informationen,
                profession.getVerbilligteSonderfertigkeiten(),
                profession.getVerbilligteVarianten() );

        // ******
        // Zauber
        // ******
        appendZauber(
                informationen,
                profession.getZauberboni() );

        // *************************
        // Empfohlene Vor-/Nachteile
        // *************************
        appendEmpfohleneVorNachteile(
                informationen,
                profession.getEmpfohleneVorteile(),
                profession.getEmpfohleneNachteile() );

        // **************************
        // Ungeeignete Vor-/Nachteile
        // **************************
        appendUngeeigneteVorNachteile(
                informationen,
                profession.getUngeeigneteVorteile(),
                profession.getUngeeigneteNachteile() );

        // *****************
        // Besonderer Besitz
        // *****************
        informationen.append( "<p><b>" );
        informationen.append( BESONDERER_BESITZ );
        informationen.append( ":</b> " );

        final List< String > besitz = profession.getBesondererBesitz();

        if ( 0 != besitz.size() ) {

            informationen.append( Strings.erzeugeAufzaehlungMitOder( besitz ) );

        } else {

            informationen.append( "-" );

        }

        informationen.append( "</p>" );

        informationen.append( "</body></html>");

        return informationen.toString();

    }

    /**
     * Aendert die Profession des Helden der gerade erschaffen wird.
     *
     * @param neueProfession des Helden.
     */
    @Override
    final void setGewaehlterBaustein( final Profession neueProfession ) {

        if ( null != neueProfession ) {

            held.setProfession( neueProfession );

        } else {

            held.setProfession( Profession.LEERE_PROFESSION );

        }
    }

    /**
     * @param professionen
     *            die entsprechend der gewaehlten Kultur moeglich sind.
     */
    void setMoeglicheProfessionen( final MoeglicheProfessionen professionen ) {

        moeglicheProfessionen = professionen;
    }

    private class ProfessionListCellRenderer extends DefaultListCellRenderer {

        @Override
        public Component getListCellRendererComponent(
                final JList list,
                final Object value,
                final int index,
                final boolean isSelected,
                final boolean cellHasFocus ) {

            final JLabel label = (JLabel) super.getListCellRendererComponent(
                    list,
                    value,
                    index,
                    isSelected,
                    cellHasFocus );

            if ( moeglicheProfessionen.istMoeglich( (Profession) value ) ) {

                label.setToolTipText( null );

            } else {

                label.setForeground( Util.FEHLERFARBE );
                label.setToolTipText( NICHT_MOEGLICH );

            }

            return label;
        }

    }

}
