/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.action;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import net.n3.nanoxml.XMLWriter;

import de.bernhardjung.dsaprogramm.bausteine.Heldenbaustein;
import de.bernhardjung.dsaprogramm.gui.DialogFehlermeldung;
import de.bernhardjung.dsaprogramm.gui.Hauptpanel;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Einstellungen;

public abstract class ActionSpeichernHeldenbaustein extends ActionSpeichern {

    /**
     * Der Text, der den Benutzer darauf hinweist, dass er ein Verzeichnis als
     * Datenquelle waehlen sollte.
     */
    private static final String DATENQUELLE_HINWEIS =
        I18n.i18n( "ActionSpeichernHeldenbaustein.DatenquelleHinweis" );

    /**
     * Der Titel fuer den Hinweisdialog ueber die Datenquelle.
     */
    private static final String DATENQUELLE_TITEL =
        I18n.i18n( "ActionSpeichernHeldenbaustein.DatenquelleHinweisTitel" );

    /** Die Option 'Datenquelle waehlen' im Hinweisdialog. */
    private static final String DATENQUELLE_WAEHLEN =
        I18n.i18n( "ActionSpeichernHeldenbaustein.DatenquelleWaehlen" );

    /** Weist, den Benutzer darauf hin, eine Id fuer den Baustein zu waehlen. */
    private static final String ID_WAEHLEN =
        I18n.i18n( "ActionSpeichernHeldenbaustein.IdWaehlen" );

    /** Die Option 'keine Datenquelle waehlen' im Hinweisdialog. */
    private static final String KEINE_DATENQUELLE_WAEHLEN =
        I18n.i18n( "ActionSpeichernHeldenbaustein.KeineDatenquelleWaehlen" );

    ActionSpeichernHeldenbaustein(
            final String nameId,
            final String mnemonicId,
            final JFrame  dasHauptfenster,
            final Hauptpanel dasHauptpanel ) {

        super( nameId, mnemonicId, dasHauptfenster, dasHauptpanel );
    }

    @Override
    public void actionPerformed( final ActionEvent e ) {

        final String id = getHeldenbausteinId();

        if ( id.equals( getIdPraefix() ) ) {

            JOptionPane.showMessageDialog( hauptfenster, ID_WAEHLEN );

            return;
        }

        // Pruefen ob Datenquelle gewaehlt wurde.
        // **************************************
        final Einstellungen einstellungen = Einstellungen.getInstanz();

        if ( ! einstellungen.istDatenquelleVerzeichnis() ) {

            final String[] optionen = { DATENQUELLE_WAEHLEN,
                                        KEINE_DATENQUELLE_WAEHLEN };

            final int ergebnis = JOptionPane.showOptionDialog(
                    hauptfenster,
                    DATENQUELLE_HINWEIS,
                    DATENQUELLE_TITEL,
                    JOptionPane.OK_CANCEL_OPTION,
                    JOptionPane.QUESTION_MESSAGE,
                    null,
                    optionen,
                    optionen[ 0 ] );

            if ( 0 == ergebnis ) {

                hauptpanel.zeigeDialogDatenquelle();

                return;
            }
        }

        super.actionPerformed( e );
    }

    @Override
    String getDateinameVorschlag() {

        final String id = getHeldenbausteinId() + ".Dateiname";

        final String name;

        if ( I18n.hatLokalisierungstext( id ) ) {

            name = I18n.i18n( id );

        } else {

            name = getHeldenbausteinName();
        }

        return name + getDateierweiterung();
    }

    /**
     * @return den Heldenbaustein, der gespeichert werden soll.
     */
    abstract Heldenbaustein getHeldenbaustein();

    /** @return Die Id des Bausteins, der gespeichert werden soll. */
    abstract String getHeldenbausteinId();

    /**
     * @return den Namen des Heldenbausteins, den der Benutzer eingegeben hat.
     */
    abstract String getHeldenbausteinName();

    /** @return "Rasse.", "Kultur." oder "Profession.". */
    abstract String getIdPraefix();

    /**
     * @return das Unterverzeichnis in der Datenquelle, in das die Bausteine
     *         gespeichert werden.
     */
    abstract String getUnterverzeichnis();

    @Override
    String getVerzeichnis() {

        final String verzeichnis;

        final Einstellungen einstellungen = Einstellungen.getInstanz();

        if ( einstellungen.istDatenquelleVerzeichnis() ) {

            final String datenquelle = einstellungen.getDatenquelle();

            verzeichnis = datenquelle + File.separator + getUnterverzeichnis();

        } else {

            verzeichnis = System.getProperty( "user.home" );
        }

        return verzeichnis;
    }

    @Override
    void setVerzeichnis( final File verzeichnis ) {
        // ignorieren. Das Verzeichnis fuer die Datenquelle wird zentral
        // angegeben.
    }

    @Override
    void speichern( final File datei ) {

        Writer out = null;

        try {

            out = new PrintWriter( datei, "UTF-8" );

            final XMLWriter xmlWriter = new XMLWriter( out );

            final Heldenbaustein baustein = getHeldenbaustein();

            xmlWriter.write( baustein.toXml(), true );

        } catch (final IOException e) {

            DialogFehlermeldung.zeigeDialog(
                    "Fehler beim schreiben der Datei.", e );

        } finally {

            if ( null != out ) {

                try {

                    out.close();

                } catch (final IOException e) {
                }
            }
        }

    }

}
