/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.bonus;

import java.awt.BorderLayout;
import java.awt.Color;
import java.util.ArrayList;
import java.util.List;

import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.gui.Util;

/**
 * Erlaubt es aus einer List von Elementen eines auszuwaehlen.
 * Die Oberflaeche besteht aus eine Liste von JRadioButton, die die Elemente
 * anzeigen die zur Auswahl stehen.
 * Weiter wird ein Text mit Hinweisen fuer den Benutzer angezeigt.
 * Der Inhalt des Panels ist in eine JScrollPane eingefasst.
 *
 * @author Bernhard Jung
 *
 * @param <T> der Typ der Elemente die zur Auswahl stehen.
 */
public class PanelAuswahl< T > extends JPanel {

    /** Die ButtonGroup fuer die Radio Buttons. */
    private final ButtonGroup buttonGroup;

    /** Die RadioButtons mit denen man die Auswahl treffen kann. */
    private final List< JRadioButton > radioButtons;

    /** Die Elemente die zur Auswahl stehen. */
    private final List< T > elemente;

    /** Erzeugt ein leeres Panel. */
    public PanelAuswahl() {

        super( new BorderLayout() );

        elemente = new ArrayList< T >();

        radioButtons = new ArrayList< JRadioButton >();

        buttonGroup = new ButtonGroup();

    }

    /**
     * Baut die Benutzeroberflaeche auf.
     *
     * @param beschreibung die dem Benutzer erklaert was zu tun ist.
     */
    public final void aufbauenBenutzeroberflaeche(
            final String beschreibung ) {

        removeAll();

        initialisiereRadioButtons();

        final JPanel panel = new JPanel();

        panel.setLayout( erzeugeLayout() );

        final CellConstraints cc = new CellConstraints();

        panel.add( new JLabel( beschreibung ), cc.xyw( 2, 2, 3 ) );

        for ( int i = 0, n = elemente.size() ; i < n ; ++i ) {

            final int y = ( 2 * i ) + 4;

            panel.add( radioButtons.get( i ), cc.xy( 3, y ) );
        }

        final JScrollPane scrollPane = new JScrollPane( panel );
        // HACK: Verhindert, dass unter GTK L&F der Hintergrund der ScrollPane
        // weiss wird. Es muss ein neues Color-Objekt erzeugt werden, wenn man
        // einfach das von getBackground() gelieferte Objekt uebernimmt, so
        // funktioniert es nicht.
        final Color color = new Color( this.getBackground().getRGB() );
        scrollPane.getViewport().setBackground( color );

        final JScrollBar scrollBar = scrollPane.getVerticalScrollBar();

        scrollBar.setUnitIncrement( Util.UNIT_INCREMENT );

        add( scrollPane, BorderLayout.CENTER );
    }

    /** Erzeugt ein Layout zur Anordnung der Button und Label. */
    private final FormLayout erzeugeLayout() {

        final StringBuilder zeilenLayout = new StringBuilder();

        zeilenLayout.append( "7dlu, P," );
        for ( int i = 0, n = elemente.size(); i < n ; ++i ) {
            zeilenLayout.append( " 4dlu, P," );
        }
        zeilenLayout.append( " 7dlu" );

        return new FormLayout( "7dlu, D:G, P, D:G, 7dlu",
                               zeilenLayout.toString() );

    }

    /**
     * @return das vom Benutzer ausgewaehlte Element.
     */
    public final T getGewaehltesElement() {

        for ( int i = 0, n = radioButtons.size(); i < n ; ++i ) {

            final JRadioButton button = radioButtons.get( i );

            if ( button.isSelected() ) {

                return elemente.get( i );
            }
        }

        // Das sollte eigentlich nicht passieren.
        throw new IllegalStateException(
                "Keiner der RadioButtons ist selektiert." );
    }

    /**
     * Sorgt dafuer das genuegend Radio Buttons vorhanden sind,
     * sie den richtigen Text haben und der erste Button selektiert ist.
     */
    private final void initialisiereRadioButtons() {

        if ( radioButtons.size() < elemente.size() ) {

            // Es gibt zu wenige Buttons es muessen mehr erzeugt werden.

            for ( int i = radioButtons.size(), n = elemente.size()
                    ; i < n
                    ; ++i ) {

                final JRadioButton button = new JRadioButton();

                buttonGroup.add( button );

                radioButtons.add( button );
            }
        }

        // Die Button mit Text versehen.
        for ( int i = 0, n = elemente.size() ; i < n ; ++i ) {

            final JRadioButton button = radioButtons.get( i );

            final T element = elemente.get( i );

            button.setText( element.toString() );
        }

        // Den ersten Button auswaehlen.
        radioButtons.get( 0 ).setSelected( true );
    }

    /**
     * Bestimmt die Elemente die zur Auswahl stehen.
     *
     * @param beschreibung die dem Benutzer angezeigt wird. Sie kann erklaeren
     *      warum oder was er waehlen soll.
     * @param dieElemente die zur Auswahl stehen. Sie werden mittels
     *      {@link Object#toString()} in Text umgewandelt um sie in der
     *      Benutzeroberflaeche anzuzeigen.
     */
    public final void setElemente(
            final String beschreibung,
            final List< T > dieElemente ) {

        elemente.clear();

        elemente.addAll( dieElemente );

        aufbauenBenutzeroberflaeche( beschreibung );

        validate();
        repaint();
    }
}
