/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.steigern;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JSpinner;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.jdesktop.layout.GroupLayout;
import org.jdesktop.layout.GroupLayout.ParallelGroup;
import org.jdesktop.layout.GroupLayout.SequentialGroup;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.gui.Lernmethoden;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.gui.VariantenListModel;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

public class PanelVorNachteileListeSteigern
extends JPanel
implements Observer {

    /** Die Aufschrift fuer den Knopf zum Abbauen eines Nachteils. */
    private static final String ABBAUEN =
        I18n.i18n( "PanelVorNachteileListeSteigern.Abbauen" );

    /**
     * Der Text mit dem die Kosten zum Abbauen eines Nachteils oder zum Senken
     * einer Schlechten Eigenschaft angezeigt werden.
     */
    private static final String AP_KOSTEN =
        I18n.i18n( "PanelVorNachteileListeSteigern.Ap" );

    /** Die Ueberschrift fuer die Spalte mit den Abbaukosten eines Nachteils. */
    private static final String KOSTEN =
        I18n.i18n( "PanelVorNachteileListeSteigern.Kosten" );

    /**
     * Die Ueberschrift fuer die Spalte mit den Checkboxen fuer die Spezielle
     * Erfahrung.
     */
    private static final String SPEZIELLE_ERFAHRUNG =
        I18n.i18n( "PanelVorNachteileListeSteigern.SpezielleErfahrung" );

    /**
     * Die Button zum Abbauen von Nachteilen.
     */
    private final Map< Integer, JButton > abbauen;

    /**
     * Die Map speichert Regelelemente (Eigenschaften, Talente usw.) als
     * Schluessel.
     * Die Listen enthalten Indizes von Vor-/Nachteilen aus {@link #werte}.
     * Die Werte in den Listen sind von den entsprechenden Vor-/Nachteilen
     * abhaengig.
     */
    private final Map< Object, List< Integer > > abhaengigkeiten;

    /**
     * Die Anzahl an Vor-/Nachteilen, die der Held beim letzten Update besessen
     * hat. Aendert sich diese Zahl, so muss die Benutzeroberflaeche neue
     * aufgebaut werden.
     */
    private int anzahlWerte;

    /**
     * Eine fette (bold) Schrift zum Markieren von vorgegebenen Vor-/Nachteilen.
     */
    private final Font fetteSchrift;

    /** Der Held dessen Werte angezeigt und manipuliert werden. */
    private Held held;

    /** Das Panel, das die ganzen Elemente der Benutzeroberflaeche beinhaltet.*/
    private final JPanel inhalt;

    /** Die Label, die die Abbaukosten anzeigen. */
    private final List< JLabel > kosten;

    /**
     * Erlaubt es die Lernmethode fuer das Abbauen von Schlechten Eigenschaften
     * zu waehlen.
     */
    private final Lernmethoden lernmethoden;

    /** Die Ueberschrift fuer die Spalte mit den Vor-/Nachteilen. */
    private final String spaltenueberschrift;

    /**
     * Speichert die Checkboxen mit denen Spezielle Erfahrungen angegeben werden
     * koennen. Der Integer-Wert steht fuer einen Wert aus {@link #werte}.
     */
    private final Map< Integer, JCheckBox > spezielleErfahrungen;

    /**
     * Speichert die Spinner aus den unterschiedlichen Zeilen. Jeder
     * Integer-Wert steht fuer einen Wert aus {@link #werte}.
     */
    private final Map< Integer, JSpinner > stufen;

    /**
     * Speichert die Comboboxen fuer die Varianten. Jeder
     * Integer-Wert steht fuer einen Wert aus {@link #werte}.
     */
    private final Map< Integer, JComboBox > varianten;

    /** Die Checkboxen zum (De)Aktivieren von Vor-/Nachteilen. */
    private final List< JCheckBox > vorNachteile;

    /** Die Werte die im Panel angezeigt werden. */
    private final List< VorNachteilWert > werte;

    /**
     * Gibt an ob das Panel die Werte darstellt, die ein Held besitzt oder eine
     * feste Liste von Werten.
     */
    private final boolean zeigtHeldenwerte;

    /**
     * @param zeigtHeldenwerte
     *            {@code true}, wenn das Panel die Werte des Helden, der mit
     *            {@link #setHeld(Held)} bestimmt wird, anzeigen soll,
     *            {@code false}, wenn das Panel eine List von Werten, die
     *            mittels {@link #setWerte(List)}, festgelegt werden, anzeigen
     *            soll.
     * @param dieLernmethoden
     *            Legt die Lernmethode zum Senken und Abbauen von Nachteilen
     *            fest.
     */
    PanelVorNachteileListeSteigern(
            final boolean zeigtHeldenwerte,
            final String dieSpaltenueberschrift,
            final Lernmethoden dieLernmethoden ) {

        abbauen = new HashMap< Integer, JButton >();

        abhaengigkeiten = new HashMap< Object, List<Integer> >();

        anzahlWerte = 0;

        fetteSchrift = getFont().deriveFont( Font.BOLD );

        inhalt = new JPanel();

        kosten = new ArrayList< JLabel >();

        lernmethoden = dieLernmethoden;
        lernmethoden.addObserver( this );

        spaltenueberschrift = dieSpaltenueberschrift;

        spezielleErfahrungen = new HashMap< Integer, JCheckBox >();

        stufen = new HashMap< Integer, JSpinner >();

        varianten = new HashMap< Integer, JComboBox >();

        vorNachteile = new ArrayList< JCheckBox >();

        werte = new ArrayList< VorNachteilWert >();

        this.zeigtHeldenwerte = zeigtHeldenwerte;

        final JScrollPane scrollPane = new JScrollPane( inhalt );
        // HACK: Verhindert, dass unter GTK L&F der Hintergrund der ScrollPane
        // weiss wird. Es muss ein neues Color-Objekt erzeugt werden, wenn man
        // einfach das von getBackground() gelieferte Objekt uebernimmt, so
        // funktioniert es nicht.
        final Color color = new Color( this.getBackground().getRGB() );
        scrollPane.getViewport().setBackground( color );

        final JScrollBar scrollBar = scrollPane.getVerticalScrollBar();

        scrollBar.setUnitIncrement( Util.UNIT_INCREMENT );

        final Dimension groesse = scrollPane.getViewport().getPreferredSize();

        groesse.height = 200;

        scrollPane.getViewport().setPreferredSize( groesse );

        setLayout( new BorderLayout() );

        add( scrollPane );

    }

    /**
     * Loescht die bisherige Benutzeroberflaeche und baut sie neu auf.
     */
    public final void aufbauenBenutzeroberflaeche() {

        final ActionListener actionAbbauen = new ActionAbbauen();

        final ActionListener actionDeAktivieren = new ActionDeAktivieren();

        final ActionListener actionSpezielleErfahrung =
            new ActionSpezielleErfahrung();

        final ChangeListener actionStufe = new ActionStufe();

        final ActionListener actionVariante ;

        if ( zeigtHeldenwerte ) {

            actionVariante = new ActionVarianteHeldenwerte();

        } else {

            actionVariante = new ActionVariante();
        }

        inhalt.removeAll();

        abbauen.clear();
        kosten.clear();
        spezielleErfahrungen.clear();
        stufen.clear();
        varianten.clear();
        vorNachteile.clear();

        final GroupLayout layout = new GroupLayout( inhalt );
        layout.setAutocreateContainerGaps( true );
        layout.setAutocreateGaps( true );

        inhalt.setLayout( layout );

        final ParallelGroup hGroup = layout.createParallelGroup();
        layout.setHorizontalGroup( hGroup );

        final SequentialGroup vGroup = layout.createSequentialGroup();
        layout.setVerticalGroup( vGroup );

        SequentialGroup zeileH = layout.createSequentialGroup();
        ParallelGroup zeileV;
        zeileV = layout.createParallelGroup( GroupLayout.BASELINE );

        JLabel ueberschrift = Util.erzeugeUeberschrift( spaltenueberschrift );
        zeileH.add( ueberschrift,
                    GroupLayout.PREFERRED_SIZE,
                    GroupLayout.PREFERRED_SIZE,
                    Short.MAX_VALUE );
        zeileV.add( ueberschrift );

        ueberschrift = Util.erzeugeUeberschrift( KOSTEN );
        zeileH.add( ueberschrift );
        zeileV.add( ueberschrift );
        final Component[] spalteKosten = new Component[ 1 + werte.size() ];
        spalteKosten[ 0 ] = ueberschrift;

        ueberschrift = Util.erzeugeUeberschrift( SPEZIELLE_ERFAHRUNG );
        zeileH.add( ueberschrift );
        zeileV.add( ueberschrift );
        final Component[] spalteSe = new Component[ 1 + werte.size() ];
        spalteSe[ 0 ] = ueberschrift;

        hGroup.add( zeileH );
        vGroup.add( zeileV );

        // Alle 5 Zeilen wird ein Trennstrich eingefuegt, das erleichtert dem
        // Benutzer die Orientierung in den Zeilen der Liste.
        final int extraZeilen = 1 + ( werte.size() / 4 );

        // Jetzt die Liste Zeile fuer Zeile aufbauen.
        for ( int i = 0, n = werte.size() + extraZeilen, trennstriche = 0
                ; i < n
                ; ++i ) {

            zeileH = layout.createSequentialGroup();
            zeileV = layout.createParallelGroup( GroupLayout.BASELINE );

            if ( ( i % 5 ) == 0 ) {
                // ein Trennstrich wird eingefuegt.

                final JSeparator separator = new JSeparator();

                zeileH.add( separator );
                zeileV.add( separator,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE );

                hGroup.add( zeileH );
                vGroup.add( zeileV );

                ++trennstriche;

                continue;
            }

            final int index = ( i - trennstriche );

            final VorNachteilWert wert = werte.get( index );

            final VorNachteil vorNachteil = wert.getVorNachteil();

            // Die Checkbox zum (De)Aktivieren und evtl. die Combobox fuer die
            // Variante.
            // ***************************************************************
            if ( vorNachteil.hatVarianten()
              && wert.istVarianteAenderbar() ) {

                final JCheckBox checkBox = new JCheckBox();

                checkBox.setText( vorNachteil.getPraefix() );

                checkBox.setName( String.valueOf( index ) );

                checkBox.addActionListener( actionDeAktivieren );

                vorNachteile.add( checkBox );

                zeileH.add( checkBox );
                zeileV.add( checkBox );

                final JComboBox comboBox =
                    new JComboBox( new VariantenListModel() );

                comboBox.setName( String.valueOf( index ) );

                comboBox.setEditable( vorNachteil.hatFreieVarianten() );

                comboBox.addActionListener( actionVariante );

                varianten.put( index, comboBox );

                zeileH.add( comboBox, 100, 100, Short.MAX_VALUE );
                zeileV.add( comboBox );

            } else {

                final JCheckBox checkBox = new JCheckBox();

                checkBox.setText( wert.getName() );

                checkBox.setName( String.valueOf( index ) );

                checkBox.addActionListener( actionDeAktivieren );

                vorNachteile.add( checkBox );

                zeileH.add( checkBox,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        Short.MAX_VALUE );
                zeileV.add( checkBox );
            }

            // Der Spinner fuer die Stufe
            // **************************
            if ( vorNachteil.hatStufen() ) {

                final JSpinner spinner = Util.erzeugeSpinner( 0, 0, 99, 1 );

                spinner.setName( String.valueOf( index ) );

                spinner.addChangeListener( actionStufe );

                stufen.put( index, spinner );

                zeileH.add( spinner,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE );
                zeileV.add( spinner );
            }

            // Die Kosten zum Abbauen eines Nachteils oder Senken einer
            // Schlechten Eigenschaft.
            // ********************************************************
            final JLabel label = new JLabel();

            spalteKosten[ 1 + index ] = label;

            kosten.add( label );

            zeileH.add( label );
            zeileV.add( label );

            hGroup.add( zeileH );
            vGroup.add( zeileV );

            // Die Checkbox fuer die Spezielle Erfahrung oder der Button zum
            // Abbauen eines Nachteils.
            // *************************************************************
            if ( ! vorNachteil.istVorteil() ) {

                if ( vorNachteil.istSchlechteEigenschaft() ) {

                    final JCheckBox se = new JCheckBox();

                    se.setName( String.valueOf( index ) );

                    se.addActionListener( actionSpezielleErfahrung );

                    spezielleErfahrungen.put( index, se );

                    spalteSe[ 1 + index ] = se;

                    zeileH.add( se );
                    zeileV.add( se );

                } else if ( ! vorNachteil.hatStufen() ) {

                    final JButton button = new JButton( ABBAUEN );

                    button.setName( String.valueOf( index ) );

                    button.addActionListener( actionAbbauen );

                    abbauen.put( index, button );

                    spalteSe[ 1 + index ] = button;

                    zeileH.add( button );
                    zeileV.add( button );

                } else {
                    final JLabel platzhalter = new JLabel();

                    spalteSe[ 1 + index ] = platzhalter;

                    zeileH.add( platzhalter );
                    zeileV.add( platzhalter );
                }
            } else {

                final JLabel platzhalter = new JLabel();

                spalteSe[ 1 + index ] = platzhalter;

                zeileH.add( platzhalter );
                zeileV.add( platzhalter );
            }
        }

        if ( 0 < werte.size() ) {

            layout.linkSize( spalteKosten, GroupLayout.HORIZONTAL );
            layout.linkSize( spalteSe,     GroupLayout.HORIZONTAL );

        }
    }

    /**
     * Initialisiert {@link #abhaengigkeiten} neu.
     */
    private void initAbhaengigkeiten() {

        abhaengigkeiten.clear();

        for ( int i = 0, n = werte.size(); i < n ; ++i ) {

            final VorNachteil vorNachteil = werte.get( i ).getVorNachteil();

            for ( final Object regelelement : vorNachteil.getAbhaengigkeiten() ) {

                setAbhaengigkeit( regelelement, i );
            }
        }
    }

    /**
     * Macht einen Wert in {@link #werte} von einem Regelelement abhaengig.
     *
     * @param regelelement
     *            von dem der Wert abhaengig ist.
     * @param index
     *            des Wertes in {@link #werte}.
     */
    private void setAbhaengigkeit(
            final Object regelelement,
            final int index ) {

        if ( ! abhaengigkeiten.containsKey( regelelement ) ) {

            abhaengigkeiten.put( regelelement, new ArrayList< Integer>() );
        }

        abhaengigkeiten.get( regelelement ).add( index );
    }

    /**
     * Bestimmt den Held, dessen Werte im Panel angezeigt werden.
     *
     * @param derHeld dessen Werte angezeigt und manipuliert werden sollen.
     */
    public final void setHeld( final Held derHeld ) {

        held = derHeld;

        if ( ! zeigtHeldenwerte ) {

            for ( int i = 0, n = werte.size() ; i < n ; ++i ) {

                final VorNachteilWert wert = werte.get( i );

                final VorNachteil vorNachteil = wert.getVorNachteil();

                if ( vorNachteil.hatStufen() ) {

                    final int min;

                    if ( held.hatVorgegeben( wert ) ) {

                        min = held.getWert( wert ).getStufe();

                    } else {

                        min = vorNachteil.getMindeststufe( held );

                    }

                    // wert.setStufe( min );

                    stufen.get( i ).setValue( min );
                }
            }
        }

        held.addObserver( this );

        anzahlWerte = -1;

        update( held, null );

        for ( int i = werte.size()-1 ; i >= 0 ; --i ) {

            update( i );
        }
    }

    /**
     * Aendert die Werte die in der Liste dargestellt werden.
     *
     * @param dieWerte die angezeigt werden.
     */
    final void setWerte( final List< VorNachteilWert > dieWerte ) {

        werte.clear();

        werte.addAll( dieWerte );

        initAbhaengigkeiten();

        aufbauenBenutzeroberflaeche();
    }

    /**
     * Aktualisiert die Zeile eines Wertes.
     *
     * @param index
     *            Der Index des Wertes.
     */
    private void update( final int index ) {

        final VorNachteilWert wert = werte.get( index );

        final VorNachteil vorNachteil = wert.getVorNachteil();

        final JCheckBox checkBox = vorNachteile.get( index );

        checkBox.setSelected( held.hat( wert ) );

        final KonfliktBeschreibung beschreibung = wert.pruefen( held );

        if ( beschreibung.enthaeltKonflikt() ) {

            checkBox.setForeground( Util.FEHLERFARBE );
            checkBox.setToolTipText( beschreibung.toHtml() );

        } else {

            checkBox.setForeground( getForeground() );
            checkBox.setToolTipText( null );

        }

        if ( held.hatVorgegeben( wert ) ) {

            checkBox.setFont( fetteSchrift );

        } else {

            checkBox.setFont( getFont() );

        }

        if ( vorNachteil.hatVarianten() && wert.istVarianteAenderbar() ) {

            final JComboBox comboBox = varianten.get( index );

            final VariantenListModel model;
            model = (VariantenListModel) comboBox.getModel();

            model.setItems( vorNachteil.getVarianten( held ) );

            comboBox.setSelectedItem( wert.getVariante() );
        }

        if ( vorNachteil.hatStufen() ) {

            final int stufe = wert.getStufe();

            final JSpinner spinner = stufen.get( index );

            spinner.setValue( stufe );
        }

        if ( abbauen.containsKey( index ) ) {

            abbauen.get( index ).setEnabled( held.hat( wert ) );

        } else if ( spezielleErfahrungen.containsKey( index ) ) {

            final JCheckBox se = spezielleErfahrungen.get( index );

            if ( held.hat( wert ) ) {

                wert.setSpezielleErfahrung(
                        held.getWert( wert ).hatSpezielleErfahrung() );
            }

            se.setSelected( wert.hatSpezielleErfahrung() );
        }

        final JLabel kostenLabel = kosten.get( index );

        if ( vorNachteil.istVorteil()
          || ( ! held.hat( wert ) ) ) {

            kostenLabel.setText( "" );

        } else {
            if ( vorNachteil.istSenkbar() ) {

                if ( held.hatVorgegeben( wert ) ) {

                    final Lernmethode lernmethode =
                        lernmethoden.getLernmethode();

                    final int apKosten = wert.getSenkungskosten( lernmethode );

                    kostenLabel.setText(
                            String.format( AP_KOSTEN, apKosten ) );

                } else {

                    kostenLabel.setText( "" );
                }

            } else {

                final int apKosten = wert.getAbbaukosten( held );

                kostenLabel.setText( String.format( AP_KOSTEN, apKosten ) );
            }
        }
    }

    /**
     * Wird aufgerufen, wenn eine Veraenderung am Helden stattgefunden hat.
     *
     * @param o ungenutzt
     * @param arg Das Regelobjekt das veraendert wurde.
     */
    public final void update( final Observable o, final Object arg ) {

        if ( zeigtHeldenwerte ) {

            final List< VorNachteilWert > heldenwerte = held.getVorNachteile();

            if ( heldenwerte.size() != anzahlWerte ) {

                anzahlWerte = heldenwerte.size();

                final List< VorNachteilWert > neueWerte =
                    held.getFehlendeVorNachteile();

                neueWerte.addAll( heldenwerte );

                setWerte( neueWerte );

                for ( int i = 0, n = werte.size() ; i < n ; ++i ) {
                    update( i );
                }

            }
        }

        if ( Lernmethoden.LERNMETHODE.equals( arg ) ) {

            for ( int i = 0, n = werte.size() ; i < n ; ++i ) {
                update( i );
            }

        } else if ( abhaengigkeiten.containsKey( arg ) ) {

            for ( final int index : abhaengigkeiten.get( arg ) ) {

                update( index );
            }
        }
    }

    private class ActionAbbauen implements ActionListener {
        public void actionPerformed( final ActionEvent e ) {

            final JButton button = (JButton) e.getSource();

            final int index = Integer.valueOf( button.getName() );

            final VorNachteilWert wert = werte.get( index );

            held.abbauen( wert );
        }
    }

    private class ActionDeAktivieren implements ActionListener {

        public void actionPerformed( final ActionEvent e ) {

            final JCheckBox checkBox = (JCheckBox) e.getSource();

            final int index = Integer.valueOf( checkBox.getName() );

            if ( checkBox.isSelected() ) {

                held.einfuegen( werte.get( index ) );

            } else {

                held.entfernen( werte.get( index ) );

            }
        }
    }

    private class ActionSpezielleErfahrung implements ActionListener {
        public void actionPerformed( final ActionEvent e ) {

            final JCheckBox checkBox = (JCheckBox) e.getSource();

            final int index = Integer.valueOf( checkBox.getName() );

            final VorNachteilWert wert = werte.get( index );

            wert.setSpezielleErfahrung( checkBox.isSelected() );

            if ( held.hat( wert ) ) {

                final VorNachteilWert heldenwert = held.getWert( wert );

                heldenwert.setSpezielleErfahrung( checkBox.isSelected() );
            }

            update( index );
        }
    }

    private class ActionStufe implements ChangeListener {

        public void stateChanged( final ChangeEvent e ) {

            final JSpinner spinner = (JSpinner) e.getSource();

            final int index = Integer.valueOf( spinner.getName() );

            final VorNachteilWert wert = werte.get( index );

            final Lernmethode lernmethode = lernmethoden.getLernmethode();

            final int stufe = (Integer) spinner.getValue();

            if ( held.hat( wert ) ) {

                held.setStufe( wert, stufe, lernmethode );

                if ( 0 == stufe ) {

                    held.abbauen( wert );
                }
            }

            wert.setStufe( stufe );

            update( index );
        }
    }

    private class ActionVariante implements ActionListener {

        public void actionPerformed( final ActionEvent e ) {

            final JComboBox comboBox = (JComboBox) e.getSource();

            Object variante = comboBox.getSelectedItem();

            if ( null == variante ) {
                // keine gueltige Variante gewaehlt. Abbruch.
                return;
            }

            if ( variante instanceof String ) {
                variante = ( (String)variante ).trim();
            }

            final int index = Integer.valueOf( comboBox.getName() );

            final VorNachteilWert wert = werte.get( index );

            // Der testwert ist da um mit  held.hat( testwert ) zu pruefen ob
            // der Held den Wert in dieser Variante schon besitzt.
            final VorNachteilWert testwert = wert.clone();

            testwert.setHeld( null );

            testwert.setVariante( variante );

            // nur wenn eine neue Variante gewaehlt wurde, ...
            if ( ! wert.getVariante().equals( variante )
                    // ... es eine sinnvolle Variante ist, ...
                    && ( ! variante.equals( "" ) ) ) {

                final VorNachteil vorNachteil = wert.getVorNachteil();

                if ( vorNachteil.hatFreieVarianten() ) {

                    vorNachteil.einfuegenVariante( variante.toString() );
                }

                wert.setVariante( variante );

            }

            update( index );
        }

    }

    private class ActionVarianteHeldenwerte implements ActionListener {

        public void actionPerformed( final ActionEvent e ) {

            final JComboBox comboBox = (JComboBox) e.getSource();

            Object variante = comboBox.getSelectedItem();

            if ( null == variante ) {
                // keine gueltige Variante gewaehlt. Abbruch.
                return;
            }

            if ( variante instanceof String ) {
                variante = ( (String)variante ).trim();
            }

            final int index = Integer.valueOf( comboBox.getName() );

            final VorNachteilWert wert = werte.get( index );

            // Der testwert ist da um mit  held.hat( testwert ) zu pruefen ob
            // der Held den Wert in dieser Variante schon besitzt.
            final VorNachteilWert testwert = wert.clone();

            testwert.setHeld( null );

            testwert.setVariante( variante );

            // nur wenn eine neue Variante gewaehlt wurde, ...
            if ( ! wert.getVariante().equals( variante )
                    // ... es eine sinnvolle Variante ist, ...
                    && ( ! variante.equals( "" ) )
                    // ... der Held die Variante noch nicht besitzt und ...
                    && ( ! held.hat( testwert ) )
                    // ... die Variante auch nicht vorgegeben ist.
                    && ( ! held.hatVorgegeben( testwert ) ) ) {

                final VorNachteil vorNachteil = wert.getVorNachteil();

                if ( vorNachteil.hatFreieVarianten() ) {

                    vorNachteil.einfuegenVariante( variante.toString() );
                }

                if ( held.hatVorgegeben( wert ) ) {

                    held.getVorgegeben( wert ).setVariante( variante );
                }

                wert.setVariante( variante );

            } else {

                update( index );
            }
        }

    }
}
