/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.util;

import java.util.ArrayList;
import java.util.List;

/**
 * Implementierung des beobachteten Objekts, des Observer-Patterns.
 * Diese Klasse ist Cloneable. Beim Klonen wird die Liste der Beobachter
 * mitgeklont. Das geklonte Objekt erhaelt seine eigene Liste von Beobachtern.
 * Die Beobachter in den Listen sind aber bei beiden Objekten die selben
 * Objekte. Wenn man {@link java.util.Observable} verwendet, haben beide Objekte
 * nach dem Klonen die selbe Liste.
 *
 * @author Bernhard Jung
 *
 */
public class Beobachtbar implements Cloneable {

    private List< Beobachter > beobachter;

    public Beobachtbar() {

        beobachter = new ArrayList< Beobachter >();
    }

    /**
     * Meldet einen Beobachter ab.
     * @param b
     *            Der Beobachter der abgemeldet werden soll.
     */
    public final void abmelden( final Beobachter b ) {

        beobachter.remove( b );
    }

    /**
     * Meldet alle Beobachter vom Objekt ab.
     */
    public final void abmeldenAlle() {
        beobachter.clear();
    }

    /**
     * @param b
     *            Ein neuer Observer, der angemeldet werden soll.
     */
    public final void anmelden( final Beobachter b ) {

        if ( ! beobachter.contains( b ) ) {

            beobachter.add( b );
        }
    }

    /**
     * @return Eine Kopie des Objekts. Die Liste der Beobachter wird dabei
     *         mitkopiert.
     */
    @Override
    public Beobachtbar clone() {

        try {

            final Beobachtbar kopie = (Beobachtbar) super.clone();

            kopie.beobachter = new ArrayList< Beobachter >( beobachter );

            return kopie;

        } catch (final CloneNotSupportedException e) {
            e.printStackTrace();
        }

        return null;
    }

    /**
     * Meldet den Beobachtern einen Veraenderung.
     *
     * @param arg
     *            Ein Objekt, das an die Beobachter gesendet wird.
     */
    protected void meldeBeobachtern( final Object arg ) {

        for ( final Beobachter b : beobachter ) {

            b.melden( this, arg );
        }
    }
}
