/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.util;

import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;

public class Einstellungen {

    private static Einstellungen EINSTELLUNGEN;

    public static final Einstellungen getInstanz() {

        if ( null == EINSTELLUNGEN ) {

            EINSTELLUNGEN = new Einstellungen();
        }

        return EINSTELLUNGEN;

    }

    private static final String DATENQUELLE = "Datenquelle";

    /**
     * Gibt bei der Datenquelle an, dass die Daten aus der JAR-Datei geladen
     * werden sollen.
     *
     * @see #getDatenquelle()
     * @see #setDatenquelle(String)
     */
    public static final String DATENQUELLE_INTERN = "Basisregeln (intern)";

    private static final String EINSTELLUNGEN_PFAD = "/de/dsachargen";

    private static final String HELDEN_RTF_VERZEICHNIS = "Helden-RTF-Verzeichnis";

    private static final String HELDEN_XML_VERZEICHNIS = "Helden-XML-Verzeichnis";

    private static final String USER_HOME = System.getProperty( "user.home" );

    private final Preferences einstellungen;

    private Einstellungen() {

        einstellungen = Preferences.userRoot().node( EINSTELLUNGEN_PFAD );
    }

    /**
     * Gibt an von wo die Daten fuer das Programm geladen werden sollen.
     *
     * @return der absolute Pfad zu einem Verzeichnis, in dem sich die Daten
     *         fuer das Programm befinden, oder {@link #DATENQUELLE_INTERN},
     *         falls der Benutzer kein solches Verzeichnis gewaehlt hat.
     */
    public final String getDatenquelle() {

        return einstellungen.get( DATENQUELLE, DATENQUELLE_INTERN );
    }

    /**
     * @return {@code true}, wenn ein Verzeichnis als Datenquelle gewaehlt
     *         worden ist, sonst {@code false}.
     */
    public final boolean istDatenquelleVerzeichnis() {
       return ! getDatenquelle().equals( DATENQUELLE_INTERN );
    }

    /**
     * Bestimmt von wo die Daten fuer das Programm geladen werden sollen.
     *
     * @param quelle
     *            der Pfad zu einem Verzeichnis, mit den Daten oder
     *            {@link #DATENQUELLE_INTERN}, falls die Daten aus der
     *            jar-Datei geladen werden sollen.
     */
    public final void setDatenquelle( final String quelle ) {

        if ( null == quelle ) {
            throw new IllegalArgumentException(
                    "null ist als Datenquelle nicht zugelassen.");
        }

        einstellungen.put( DATENQUELLE, quelle );

        try {
            einstellungen.flush();
        } catch (final BackingStoreException e) {
        }
    }

    /**
     * @return der Pfad zum Verzeichnis in dem die RTF-Dateien der Helden
     *         gespeichert werden.
     */
    public final String getHeldenRtfVerzeichnis() {

        return einstellungen.get( HELDEN_RTF_VERZEICHNIS, USER_HOME );
    }

    /**
     * @param pfad
     *            der Pfad zum Verzeichnis in dem die RTF-Dateien der Helden
     *            gespeichert werden.
     */
    public final void setHeldenRtfVerzeichnis( final String pfad ) {

        if ( null == pfad ) {
            throw new IllegalArgumentException(
                    "null ist kein gueltiger Pfad." );
        }

        einstellungen.put( HELDEN_RTF_VERZEICHNIS, pfad );

        try {
            einstellungen.flush();
        } catch (final BackingStoreException e) {
        }
    }

    /**
     * @return der Pfad zum Verzeichnis in dem die XML-Dateien der Helden
     *         gespeichert werden.
     */
    public final String getHeldenXmlVerzeichnis() {

        return einstellungen.get( HELDEN_XML_VERZEICHNIS, USER_HOME );
    }

    /**
     * @param pfad
     *            der Pfad zum Verzeichnis in dem die XML-Dateien der Helden
     *            gespeichert werden.
     */
    public final void setHeldenXmlVerzeichnis( final String pfad ) {

        if ( null == pfad ) {
            throw new IllegalArgumentException(
                    "null ist kein gueltiger Pfad." );
        }

        einstellungen.put( HELDEN_XML_VERZEICHNIS, pfad );

        try {
            einstellungen.flush();
        } catch (final BackingStoreException e) {
        }
    }
}
