/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.vornachteile;

import java.util.ArrayList;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.Held;

/**
 * Speichert eine Liste von Vor-/Nachteilen, zB alle Vor-/Nachteile die ein
 * Held besitzt.
 *
 * @author Bernhard Jung
 */
public class VorNachteilWerte {

    /** Der Held dem die Vor-/Nachteile gehoeren. */
    private final Held held;

    /** Die Liste der Vor-/Nachteile. */
    private final List< VorNachteilWert > vorNachteile;

    /** Erzeugt eine neue Liste fuer Vor-/Nachteile. */
    public VorNachteilWerte() {
        this( null );
    }

    public VorNachteilWerte( final Held derHeld ) {

        held = derHeld;

        vorNachteile = new ArrayList< VorNachteilWert >();
    }

    /**
     * Entfernt einen Vor-/Nachteil aus der Liste. Wenn der Vor-/Nachteil nicht
     * in der Liste ist, so macht die Methode nichts. Nach Aufruf der Methode
     * sind alle Varianten des Vor-/Nachteils entfernt.
     *
     * @param vorNachteil
     *            der aus der Liste entfernt werden soll.
     * @throws NullPointerException
     *             Kann geworfen werden wenn {@code null} uebergeben wird.
     */
    public final void entfernen( final VorNachteil vorNachteil ) {

        for ( int i = vorNachteile.size() - 1 ; i >= 0 ; --i ) {

            final VorNachteil anderer = vorNachteile.get( i ).getVorNachteil();

            if ( vorNachteil.equals( anderer ) ) {

                vorNachteile.remove( i );
            }
        }
    }

    /**
     * Entfernt einen Wert aus der Liste. Wenn der Wert nicht in der Liste ist,
     * so macht die Methode nichts.
     *
     * @param wert
     *            der aus der Liste entfernt werden soll.
     * @return den entfernten Wert.
     * @throws NullPointerException
     *             Kann geworfen werden wenn {@code null} uebergeben wird.
     */
    public final VorNachteilWert entfernen( final VorNachteilWert wert ) {

        for ( int i = vorNachteile.size() - 1 ; i >= 0 ; --i ) {

            if ( wert.equals( vorNachteile.get( i ) ) ) {

                return vorNachteile.remove( i );
            }
        }

        return null;
    }

    /**
     * Prueft ob ein Vor-/Nachteil in der Liste vorhanden ist.
     *
     * @param vorNachteil
     *            fuer den geprueft werden soll ob er in der Liste ist.
     * @return {@code true}, wenn der Vor-/Nachteil in der Liste ist, sonst
     *         {@code false}.
     */
    public final boolean enthaelt( final VorNachteil vorNachteil ) {

        for ( final VorNachteilWert wert : vorNachteile ) {

            if ( wert.getVorNachteil().equals( vorNachteil ) ) {
                return true;
            }
        }

        return false;
    }

    /**
     * Prueft ob ein Wert in der Liste vorhanden ist.
     *
     * @param wert
     *            fuer den geprueft werden soll ob er in der Liste ist.
     * @return {@code true}, wenn der Wert in der Liste ist, sonst
     *         {@code false}.
     */
    public final boolean enthaelt( final VorNachteilWert wert ) {
        return vorNachteile.contains( wert );
    }

    /**
     * Fuegt einen neuen Wert in die Liste ein. Wenn der Wert schon in der Liste
     * ist, so macht die Methode nichts. Ist der Wert schon in der Liste und
     * besitzt der Vor-/Nachteil Stufen, so wird die Stufe des Wertes zu Stufe
     * des Wertes in der Liste addiert.
     *
     * @param wert
     *            der in dei Liste eingefuegt werden soll.
     */
    public final void einfuegen( final VorNachteilWert wert ) {

        final VorNachteil neu = wert.getVorNachteil();

        if ( ! enthaelt( wert ) ) {

            // Es wird von hinten nach vorne durch die Liste der erste
            // Vor-/Nachteil gesucht der kleiner gleich des neue Vor-/Nachteils
            // ist. Hinter dem wird der neue Vor-/Nachteil dann eingefuegt.
            int i = vorNachteile.size() - 1;

            for ( ; i >= 0 ; --i ) {

                final VorNachteil vorNachteil;
                vorNachteil = vorNachteile.get( i ).getVorNachteil();

                if ( neu.compareTo( vorNachteil ) >= 0 ) {
                    break;
                }
            }

            ++i;

            final VorNachteilWert kopie = wert.clone();

            kopie.setHeld( held );

            vorNachteile.add( i, kopie );

            if ( null != held ) {
                held.notifyObservers( neu );
            }

        } else if ( neu.hatStufen() ) {

            final int stufe = wert.getStufe();

            getWert( wert ).addiereStufe( stufe );

            if ( null != held ) {
                held.notifyObservers( neu );
            }
        }
    }

    /**
     * Liefert den zu einem Vor-/Nachteil gehoerenden Wert aus der Liste.
     *
     * @param vorNachteil
     *            dessen Wert gesucht ist.
     * @return einen zum uebergebenen Vor-/Nachteil gehoerenden Wert. Sind in
     *         der Liste mehrere Varianten des Vor-/Nachteils gespeichert, so
     *         wird eine beliebige davon zurueckgegeben. Gib es keinen passenden
     *         Wert, so wird {@code null} geliefert.
     */
    public final VorNachteilWert getWert( final VorNachteil vorNachteil ) {

        for ( final VorNachteilWert wert : vorNachteile ) {

            if ( wert.getVorNachteil().equals( vorNachteil ) ) {

                return wert;
            }
        }

        return null;
    }

    /**
     * Liefert einen bestimmten Wert aus der Liste. Es handelt sich um keine
     * Kopie des Wertes. Wenn der Wert veraendert wird aendert sich der Wert in
     * der Liste.
     *
     * @param wert
     *            der gesucht ist.
     * @return den gesuchten Wert oder {@code null} wenn der Wert nicht in der
     *         Liste vorhanden ist.
     */
    public final VorNachteilWert getWert( final VorNachteilWert wert ) {

        for ( final VorNachteilWert w : vorNachteile ) {

            if ( wert.equals( w ) ) {

                return w;
            }
        }

        return null;
    }

    /**
     * @return eine Liste der Werte die in dieser VorNachteile-Liste gespeichert
     *         sind. Es handelt sich um die Objekte aus der Liste, keine Kopien.
     *         Aenderungen an ihnen aendern die Werte in der VorNachteile-Liste.
     */
    public final List< VorNachteilWert > getWerte() {
        return new ArrayList< VorNachteilWert >( vorNachteile );
    }

    /**
     * Liefert eine Liste aller Varianten eines Vor-/Nachteils.
     *
     * @param vorNachteil
     *            dessen Varianten gesucht sind.
     * @return eine Liste aller Varianten vom Vor-/Nachteil {@code vorNachteil}.
     */
    public final List< VorNachteilWert > getWerte(
            final VorNachteil vorNachteil ) {

        final List< VorNachteilWert > werte;
        werte = new ArrayList< VorNachteilWert >();

        for ( final VorNachteilWert wert : vorNachteile ) {

            if ( wert.getVorNachteil().equals( vorNachteil ) ) {

                werte.add( wert );
            }
        }

        return werte;
    }

}
