/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.zauber;

import java.text.Collator;
import java.util.AbstractList;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import de.bernhardjung.dsaprogramm.i18n.I18n;

/**
 * Wie eine Talentgruppe ist auch eine Zauberguppe eine Liste von Zaubern.
 *
 * @author Bernhard Jung
 *
 */
public class Zaubergruppe
extends AbstractList< Zauber >
implements Comparable< Zaubergruppe > {

    /** Eine Liste aller Zaubergruppen. */
    private static final List< Zaubergruppe > GRUPPEN;

    /** Die normalen Zauber, fuer Elfen und Magier. */
    public static final Zaubergruppe ZAUBER =
        new Zaubergruppe( "Zaubergruppe.Zauber" );

    /** Die Zauber fuer den Vorteil uebernatuerliche Begabung. */
    public static final Zaubergruppe BEGABUNG =
        new Zaubergruppe( "Zaubergruppe.UebernatuerlicheBegabungen" );

    static {

        GRUPPEN = new ArrayList< Zaubergruppe >( 2 );

        GRUPPEN.add(  ZAUBER );
        GRUPPEN.add( BEGABUNG );

        Collections.sort( GRUPPEN );
    }

    /**
     * @return eine sortierte Liste aller Zaubergruppen.
     */
    public static final List< Zaubergruppe > getZaubergruppen() {
        return new ArrayList< Zaubergruppe >( GRUPPEN );
    }

    /** Die Id der Gruppe */
    private final String id;

    /** Eine kuerzere Form des Namens der Gruppe. */
    private final String kuerzel;

    /** Der volle Name der Gruppe. */
    private final String name;

    /** Die Zauber die zu der Gruppe gehoeren. */
    private final List< Zauber > zauber;

    /**
     * @param dieId der Gruppe.
     */
    private Zaubergruppe( final String dieId ) {

        id = dieId;

        kuerzel = I18n.i18n( id + ".Kuerzel" );

        name = I18n.i18n( id + ".Name" );

        zauber = new ArrayList< Zauber >();
    }

    /** Sortiert die Zaubergruppen alphabtisch nach Namen. */
    public int compareTo( final Zaubergruppe o ) {
        return Collator.getInstance().compare( getName(), o.getName() );
    }

    /**
     * Fuegt einen neuen Zauber in die Gruppe ein.
     * Macht nichts, wenn der Zauber schon in der Liste ist.
     *
     * @param neuerZauber fuer die Gruppe.
     */
    final void einfuegen( final Zauber neuerZauber ) {

        int index = Collections.binarySearch( zauber, neuerZauber );

        if ( 0 > index ) {

            index = -( index + 1 );

            zauber.add( index, neuerZauber );
        }
    }

    /**
     * @return die Kurzform des Namens.
     */
    public final String getKuerzel() {
        return kuerzel;
    }

    /**
     * @return einen Zauber aus der Liste.
     */
    @Override
    public final Zauber get( final int index ) {
        return zauber.get( index );
    }

    /**
     * @return Den Namen der Gruppe.
     */
    public final String getName() {
        return name;
    }

    /**
     * @return die Anzahl von Zaubern in der Gruppe.
     */
    @Override
    public final int size() {
        return zauber.size();
    }
}
